/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2019-2020. All rights reserved.
 * Description: Header of Det Module.
 * Author: VOS
 * Create: 2019-08-01
 *
 */

#ifndef DET_H
#define DET_H

#include "Std_Types.h"
#include "Det_Cfg.h"

/****************************************************************************************
 *  Macro Definitions
 ****************************************************************************************/
/* for AUTOSAR info */
#define DET_AR_RELEASE_MAJOR_VERSION 4
#define DET_AR_RELEASE_MINOR_VERSION 4

/* for module version info */
#define DET_SW_MAJOR_VERSION        3
#define DET_SW_MINOR_VERSION        2
#define DET_SW_PATCH_VERSION        224

#define DET_VENDOR_ID               PLATFORM_VENDOR_ID
#define DET_MODULE_ID               15
#define DET_INSTANCE_ID             0

#define DET_GET_VERSION_INFO        0
#define DET_E_PARAM_POINTER         1

/**********************************************************************************************************************
 * Version Check
 *********************************************************************************************************************/
#if (!defined(DET_CFG_MAJOR_VERSION) || (DET_CFG_MAJOR_VERSION != DET_SW_MAJOR_VERSION))
#error CFG major Version not defined or mismatch.
#endif

#if (!defined(DET_CFG_MINOR_VERSION) || (DET_CFG_MINOR_VERSION != DET_SW_MINOR_VERSION))
#error CFG minor version not defined or mismatch.
#endif

#if (!defined(DET_CFG_PATCH_VERSION) || (DET_CFG_PATCH_VERSION != DET_SW_PATCH_VERSION))
#error CFG patch version not defined or mismatch.
#endif

/****************************************************************************************
 *  Struct Declaration
 ****************************************************************************************/
typedef struct {
    /* The ModuleId of the reporting module */
    uint16 ModuleId;

    /* The InstanceId of the reporting module */
    uint8 InstanceId;

    /* The ApiId of the reporting function */
    uint8 ApiId;

    /* The ErrorId of the reporting error */
    uint8 ErrorId;
} Det_ErrorInfo;

typedef struct {
    /* Det Module does not need this config, so use a dummy one */
    uint8 Det_ConfigNeverUsed;
} Det_ConfigType;
/** @} */

/****************************************************************************************
 *  Global Variable Declaration
 ****************************************************************************************/
#if (DET_TESTING_MODE == STD_ON)
extern volatile Det_ErrorInfo Det_Error;
#endif

/**********************************************************************************************************************
 *  Function Declaration
 *********************************************************************************************************************/
#define DET_START_SEC_SHARE_CODE
#include "Det_MemMap.h"

/**
 * Service to initialize the Default Error Tracer.
 *
 *
@verbatim
  serviceID{0x00}
  synchronous{true}
  reentrant{false}
@endverbatim
 *
 * @req{ SAR-VOS-DET-032-2210-0001 ,
 * Det shall supports initialization of internal data,
 * D,
 * SSR-VOS-DET-032-2210-0001
 * }
 *
 * @param [in] ConfigPtr Pointer to config.
 * @param [out] none
 * @return none.
 */
FUNC(void, DET_CODE) Det_Init(CONSTP2CONST(Det_ConfigType, AUTOMATIC, DET_CONST) ConfigPtr);
/**
 * Service to start the Default Error Tracer.
 *
@verbatim
  serviceID{0x02}
  synchronous{true}
  reentrant{true}
@endverbatim
 *
 * @req{ SAR-VOS-DET-032-2210-0001 ,
 * Det shall supports initialization of internal data,
 * D,
 * SSR-VOS-DET-032-2210-0001
 * }
 *
 * @param [in] none.
 * @param [out] none.
 * @return none.
 */
FUNC(void, DET_CODE) Det_Start(void);
/**
 * Service to report development errors.
 *
@verbatim
  serviceID{0x01}
  synchronous{true}
  reentrant{true}
@endverbatim
 * @req{ SAR-VOS-DET-032-2210-0006,
   Det shall supports the callback interface for reporting development errors,
 * D,
 * SSR-VOS-DET-032-2210-0003
 * }
 *
*  @req{ SAR-VOS-DET-032-2210-0005,
   Det shall supports reporting development errors,
 * D,
 * SSR-VOS-DET-032-2210-0003
 * }
 *
 * @param [in] ModuleId Module ID of calling module.
 * @param [in] InstanceId The identifier of the index based instance of a module, starting from 0,
   If the module is a single instance module it shall pass 0 as the InstanceId.
 * @param [in] ApiId ID of API service in which transient fault is detected (defined in SWS of calling module)
 * @param [in] ErrorId ID of detected develop error (defined in SWS of calling module).
 * @param [out] none.
 * @return Error code of the function.
 * @retval  E_OK The service has succeeded.
 * @retval  E_NOT_OK call of the function failed.
 */
FUNC(Std_ReturnType, DET_CODE) Det_ReportError(VAR(uint16, AUTOMATIC) ModuleId,
    VAR(uint8, AUTOMATIC) InstanceId, VAR(uint8, AUTOMATIC) ApiId, VAR(uint8, AUTOMATIC) ErrorId);
/**
 * Service to report runtime errors. If a callout has been configured then this callout shall be called.
 *
@verbatim
  serviceID{0x04}
  synchronous{true}
  reentrant{true}
@endverbatim
 * @req{ SAR-VOS-DET-032-2210-0003,
   Det shall supports reporting runtime errors,
 * D,
 * SSR-VOS-DET-032-2210-0002
 * }
 *
 * @req{ SAR-VOS-DET-032-2210-0004,
   Det shall supports the callback interface for reporting runtime errors,
 * D,
 * SSR-VOS-DET-032-2210-0002
 * }
 *
 * @param [in] ModuleId Module ID of calling module.
 * @param [in] InstanceId The identifier of the index based instance of a module, starting from 0,
   If the module is a single instance module it shall pass 0 as the InstanceId.
 * @param [in] ApiId ID of API service in which transient fault is detected (defined in SWS of calling module)
 * @param [in] ErrorId ID of detected runtime error (defined in SWS of calling module).
 * @param [out] none.
 * @return Error code of the function.
 * @retval  E_OK The service has succeeded.
 * @retval  E_NOT_OK call of the function failed.
 */
FUNC(Std_ReturnType, DET_CODE) Det_ReportRuntimeError(VAR(uint16, AUTOMATIC) ModuleId,
    VAR(uint8, AUTOMATIC) InstanceId, VAR(uint8, AUTOMATIC) ApiId, VAR(uint8, AUTOMATIC) ErrorId);
/**
 * Service to report transient faults. If a callout has been configured than this callout shall be called
   and the returned value of the callout shall be returned. Otherwise it returns immediately with E_OK..
 *
@verbatim
  serviceID{0x05}
  synchronous{true}
  reentrant{true}
@endverbatim
 * @req{ SAR-VOS-DET-032-2210-0007,
   Det shall supports the mechanism of reporting transient errors,
 * D,
 * SSR-VOS-DET-032-2210-0004
 * }
 *
 * @req{SAR-VOS-DET-032-2210-0008,
   Det shall supports the callback interface for reporting transient errors
 * D,
 * SSR-VOS-DET-032-2210-0004
 * }
 *
 * @param [in] ModuleId Module ID of calling module.
 * @param [in] InstanceId The identifier of the index based instance of a module, starting from 0,
   If the module is a single instance module it shall pass 0 as the InstanceId.
 * @param [in] ApiId ID of API service in which transient fault is detected (defined in SWS of calling module)
 * @param [in] FaultId ID of detected transient fault (defined in SWS of calling module).
 * @param [out] none.
 * @return Error code of the function.
 * @retval  E_OK The service has succeeded.
 * @retval  E_NOT_OK call of the function failed.
 */
FUNC(Std_ReturnType, DET_CODE) Det_ReportTransientFault(VAR(uint16, AUTOMATIC) ModuleId,
    VAR(uint8, AUTOMATIC) InstanceId, VAR(uint8, AUTOMATIC) ApiId, VAR(uint8, AUTOMATIC) FaultId);

#if (DET_VERSION_INFO_API == STD_ON)
/**
 * Returns the version information of this module.
 *
@verbatim
  serviceID{0x03}
  synchronous{true}
  reentrant{true}
@endverbatim
 * @req{ SAR-VOS-DET-032-2210-0002,
   Det shall supports obtaining version information,
 * D,
 * SSR-VOS-DET-032-2210-0001
 * }
 *
 * @param [in] none.
 * @param [out] versioninfo Pointer to where to store the version information of this module.
 * @return none.
 */
FUNC(void, DET_CODE) Det_GetVersionInfo(P2VAR(Std_VersionInfoType, AUTOMATIC, DET_APPL_DATA) versioninfo);
#endif

#define DET_STOP_SEC_SHARE_CODE
#include "Det_MemMap.h"
#endif /* DET_H */
