/**************************************************************************************
 * Copyright (c) Huawei Technologies Co., Ltd. 2019-2023. All rights reserved.
 * Description: Rte_Util.h
 * Author: VOS OS
 * Create: 2019-12-31
 *
 **************************************************************************************/
#ifndef RTE_UTIL_H
#define RTE_UTIL_H

#define RTE_UTIL_AR_RELEASE_MAJOR_VERSION 4U
#define RTE_UTIL_AR_RELEASE_MINOR_VERSION 4U

#include "Rte.h"
#include "Rte_Type.h"
/* Version verification of the Rte_Type module */
#if ((RTE_TYPE_AR_RELEASE_MAJOR_VERSION != 4U) || (RTE_TYPE_AR_RELEASE_MINOR_VERSION != 4U))
#error The imported RTE_TYPE module version is incompatible, there may be problems.
#endif
#include "qn_libcutil.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/*
 * stack type
 *
 * stackPtr: stack corresponds the first address of array
 * top: the top index +1
 * stat: stack state
 * stackSize: length of stack
 * elementSize: bytes of each element in stack
 */
#if !defined(RTE_TYPE_RteStackSizeType)
#define RTE_TYPE_RteStackSizeType
typedef uint32 RteStackSizeType;
typedef uint32 RteStackElementSizeType32;
#endif

typedef struct {
    P2VAR(void, AUTOMATIC, RTE_CODE) stackPtr;
    VAR(RteStackSizeType, AUTOMATIC) top;
    VAR(Std_ReturnType, AUTOMATIC) stat;
    VAR(RteStackSizeType, AUTOMATIC) stackSize;
    VAR(RteStackElementSizeType32, AUTOMATIC) elementSize;
} RteStackType;

/*
 * return enqueue operation error code.
 * queue: specified global queue, src: address of new data to be enqueued.
 * enqueue: copy the beginning elementSize bytes data of src address to the tail of queue.
 */
#define RTE_START_SEC_CODE
#include "Rte_MemMap.h"
static inline VAR(Std_ReturnType, AUTOMATIC) Rte_Enqueue(
    P2VAR(RteQueueType, AUTOMATIC, RTE_APPL_DATA) queue,
    P2CONST(void, AUTOMATIC, RTE_APPL_DATA) src,
    VAR(RteQueueElementSizeType32, AUTOMATIC) srcSize)
{
    Std_ReturnType result;
    if (queue == NULL_PTR) {
        result = RTE_E_LOST_DATA;
    } else {
        if ((queue->elementSize != srcSize) || (src == NULL_PTR)) {
            /*
             * The size of the input element does not match that of the queue element.
             */
            queue->stat |= RTE_E_LOST_DATA;
        } else if (queue->elementCount == queue->elementArraySize) {
            /*
             * stat: RTE_QUEUE_STATUS_LOST_DATA
             */
            queue->stat |= RTE_E_LOST_DATA;
        } else {
            /* if tail > elementArraySize, move to begin */
            P2VAR(void, AUTOMATIC, RTE_APPL_DATA) dest = &(((ByteType8 *)(queue->elementArrayPtr))[(queue->elementSize) * ((queue->front + queue->elementCount) % (queue->elementArraySize))]);
            (void)qn_memcpy(dest, src, srcSize);
            ++queue->elementCount;
            queue->stat = RTE_E_OK;
        }
        result = queue->stat;
    }
    return result;
}
#define RTE_STOP_SEC_CODE
#include "Rte_MemMap.h"

/*
 * return dequeue operation error code.
 * queue: specified global queue, dest: address of var memory to contain data dequeued from queue.
 * dequeue: copy the beginning elementSize bytes data of queue header address to dest.
 */
#define RTE_START_SEC_CODE
#include "Rte_MemMap.h"
static inline VAR(Std_ReturnType, AUTOMATIC) Rte_Dequeue(
    P2VAR(RteQueueType, AUTOMATIC, RTE_APPL_DATA) queue,
    P2VAR(void, AUTOMATIC, RTE_APPL_DATA) dest,
    VAR(RteQueueElementSizeType32, AUTOMATIC) srcSize)
{
    Std_ReturnType result;
    if (queue == NULL_PTR) {
        result = RTE_E_NO_DATA;
    } else {
        if ((queue->elementSize != srcSize) || (dest == NULL_PTR)) {
            /*
             * The size of the input element does not match that of the queue element.
             */
            queue->stat = RTE_E_NO_DATA;
        } else if (queue->elementCount > 0U) {
            P2CONST(void, AUTOMATIC, RTE_APPL_DATA) src = &(((ByteType8 *)(queue->elementArrayPtr))[(queue->elementSize) * ((queue->front) % (queue->elementArraySize))]);
            (void)qn_memcpy(dest, src, srcSize);
            --queue->elementCount;
            ++queue->front;
            queue->stat = RTE_E_OK;
        } else {
            queue->stat = RTE_E_NO_DATA;
        }
        result = queue->stat;
    }
    return result;
}
#define RTE_STOP_SEC_CODE
#include "Rte_MemMap.h"

/*
 * return save cache queue data error code.
 * clearqueue: clear the queue.
 */
#define RTE_START_SEC_CODE
#include "Rte_MemMap.h"
static inline VAR(uint8, AUTOMATIC) Rte_EnqueueForProxy(
    P2VAR(void, AUTOMATIC, RTE_APPL_DATA) cacheSendQueueData,
    P2VAR(RteCacheDataSendInfo, AUTOMATIC, RTE_APPL_DATA) cacheSendQueueDataInfo,
    P2VAR(RteQueueType, AUTOMATIC, RTE_APPL_DATA) queue,
    P2CONST(void, AUTOMATIC, RTE_APPL_DATA) src,
    VAR(uint32, AUTOMATIC) srcSize)
{
    Std_ReturnType result;
    if ((cacheSendQueueData == NULL_PTR) || (cacheSendQueueDataInfo == NULL_PTR) || (queue == NULL_PTR)) {
        result = RTE_E_LOST_DATA;
    } else if (queue->elementCount == queue->elementArraySize) {
        result = RTE_E_LOST_DATA;
    } else if(srcSize > cacheSendQueueDataInfo->elementSize) {
        result = RTE_E_LOST_DATA;
    } else {
        if (cacheSendQueueDataInfo->saveIndex == queue->elementArraySize) {
            cacheSendQueueDataInfo->saveIndex = 0;
        }
        P2VAR(void, AUTOMATIC, RTE_APPL_DATA) dest = &(((uint8 *)(cacheSendQueueData))[(cacheSendQueueDataInfo->elementSize) * ((cacheSendQueueDataInfo->saveIndex) % (queue->elementArraySize))]);
        (void) qn_memset(dest, 0, cacheSendQueueDataInfo->elementSize);
        (void) qn_memcpy(dest, src, srcSize);
        cacheSendQueueDataInfo->thisIndex = cacheSendQueueDataInfo->saveIndex;
        ++cacheSendQueueDataInfo->saveIndex;
        result = RTE_E_OK;
    }
    return result;
}
#define RTE_STOP_SEC_CODE
#include "Rte_MemMap.h"

#define rte_lock_osa_OSA_C0_1_HW_QM() \
        ( \
            (void) GetResource(RTE_OSA_OSA_C0_1_HW_QM) \
        )
#define rte_unlock_osa_OSA_C0_1_HW_QM() \
        ( \
            (void) ReleaseResource(RTE_OSA_OSA_C0_1_HW_QM) \
        )

#define rte_lock_osa_OSA_C0_3_HW_ASILD() \
        ( \
            (void) GetResource(RTE_OSA_OSA_C0_3_HW_ASILD) \
        )
#define rte_unlock_osa_OSA_C0_3_HW_ASILD() \
        ( \
            (void) ReleaseResource(RTE_OSA_OSA_C0_3_HW_ASILD) \
        )

#define rte_lock_osa_OSA_C1_1_APP_QM() \
        ( \
            (void) GetResource(RTE_OSA_OSA_C1_1_APP_QM) \
        )
#define rte_unlock_osa_OSA_C1_1_APP_QM() \
        ( \
            (void) ReleaseResource(RTE_OSA_OSA_C1_1_APP_QM) \
        )

#define rte_lock_osa_OSA_C1_2_APP_ASILD() \
        ( \
            (void) GetResource(RTE_OSA_OSA_C1_2_APP_ASILD) \
        )
#define rte_unlock_osa_OSA_C1_2_APP_ASILD() \
        ( \
            (void) ReleaseResource(RTE_OSA_OSA_C1_2_APP_ASILD) \
        )

#define rte_lock_osa_OSA_C2_1_APP_QM() \
        ( \
            (void) GetResource(RTE_OSA_OSA_C2_1_APP_QM) \
        )
#define rte_unlock_osa_OSA_C2_1_APP_QM() \
        ( \
            (void) ReleaseResource(RTE_OSA_OSA_C2_1_APP_QM) \
        )

#define rte_lock_osa_OSA_C2_2_APP_ASILD() \
        ( \
            (void) GetResource(RTE_OSA_OSA_C2_2_APP_ASILD) \
        )
#define rte_unlock_osa_OSA_C2_2_APP_ASILD() \
        ( \
            (void) ReleaseResource(RTE_OSA_OSA_C2_2_APP_ASILD) \
        )

#define rte_lock_osa_OSA_C3_1_APP_QM() \
        ( \
            (void) GetResource(RTE_OSA_OSA_C3_1_APP_QM) \
        )
#define rte_unlock_osa_OSA_C3_1_APP_QM() \
        ( \
            (void) ReleaseResource(RTE_OSA_OSA_C3_1_APP_QM) \
        )

#define rte_lock_osa_OSA_C3_2_APP_ASILD() \
        ( \
            (void) GetResource(RTE_OSA_OSA_C3_2_APP_ASILD) \
        )
#define rte_unlock_osa_OSA_C3_2_APP_ASILD() \
        ( \
            (void) ReleaseResource(RTE_OSA_OSA_C3_2_APP_ASILD) \
        )

#define rte_lock_osa_OSA_C4_1_APP_QM() \
        ( \
            (void) GetResource(RTE_OSA_OSA_C4_1_APP_QM) \
        )
#define rte_unlock_osa_OSA_C4_1_APP_QM() \
        ( \
            (void) ReleaseResource(RTE_OSA_OSA_C4_1_APP_QM) \
        )

#define rte_lock_osa_OSA_C5_1_APP_QM() \
        ( \
            (void) GetResource(RTE_OSA_OSA_C5_1_APP_QM) \
        )
#define rte_unlock_osa_OSA_C5_1_APP_QM() \
        ( \
            (void) ReleaseResource(RTE_OSA_OSA_C5_1_APP_QM) \
        )

#define isQueueEmpty(a) \
        ( \
            ((a).counter == 0U) \
        )

#define isQueueFull(a, dim) \
        ( \
            ((a).counter == (dim)) \
        )

#define insertQueue(a, dim, b) \
        ( \
            isQueueFull(a, dim) ? \
                FALSE : ( \
                (a).queue[(a).tail] = (b), \
                (a).counter += 1U, \
                (a).tail = ((a).tail + 1U) % (dim), \
                TRUE ) \
        )

#define insertQueueArray(a, dim, b, size) \
        ( \
            isQueueFull(a, dim) ? \
                FALSE : ( \
                (void) qn_memcpy(&((a).queue[(a).tail]), (b), (size)), \
                (a).counter += 1U, \
                (a).tail = ((a).tail + 1U) % (dim), \
                TRUE ) \
        )

#define removeQueue(a, dim, result) \
        ( \
            isQueueEmpty(a) ? \
                FALSE : ( \
                (result) = (a).queue[(a).head], \
                (a).counter -= 1U, \
                (a).head = ((a).head + 1U) % (dim), \
                TRUE ) \
        )

#define removeQueueArray(a, dim, result, size) \
        ( \
            isQueueEmpty(a) ? \
                FALSE : ( \
                (void) qn_memcpy((result), &((a).queue[(a).head]), (size)), \
                (a).counter -= 1U, \
                (a).head = ((a).head + 1U) % (dim), \
                TRUE ) \
        )

#define RTE_START_SEC_CODE
#include "Rte_MemMap.h"
static inline VAR(Std_ReturnType, AUTOMATIC) Rte_StackPush(
    P2VAR(RteStackType, AUTOMATIC, RTE_APPL_DATA) stack,
    P2CONST(void, AUTOMATIC, RTE_APPL_DATA) src,
    VAR(RteStackElementSizeType32, AUTOMATIC) srcSize)
{
    Std_ReturnType result;
    if (stack == NULL_PTR) {
        result = RTE_E_LOST_DATA;
    } else {
        if ((stack->elementSize != srcSize) || (src == NULL_PTR)) {
            /*
             * The size of the input element does not match that of the queue element.
             */
            stack->stat |= RTE_E_LOST_DATA;
        } else if (stack->top == stack->stackSize) {
            /*
             * stat: RTE_STACK_STATUS_LOST_DATA
             */
            stack->stat |= RTE_E_LOST_DATA;
        } else {
            P2VAR(void, AUTOMATIC, RTE_APPL_DATA) dest = &(((ByteType8 *)(stack->stackPtr))[(stack->elementSize) * (stack->top)]);
            (void)qn_memcpy(dest, src, srcSize);
            ++stack->top;
            stack->stat = RTE_E_OK;
        }
        result = stack->stat;
    }
    return result;
}
#define RTE_STOP_SEC_CODE
#include "Rte_MemMap.h"

#define RTE_START_SEC_CODE
#include "Rte_MemMap.h"
static inline VAR(Std_ReturnType, AUTOMATIC) Rte_StackPop(
    P2VAR(RteStackType, AUTOMATIC, RTE_APPL_DATA) stack,
    VAR(RteStackElementSizeType32, AUTOMATIC) destSize)
{
    Std_ReturnType result;
    if (stack == NULL_PTR) {
        result = RTE_E_LOST_DATA;
    } else {
        if (stack->elementSize != destSize) {
            /*
             * The size of the input element does not match that of the queue element.
             */
            stack->stat |= RTE_E_LOST_DATA;
        } else if (stack->top > 0U) {
            --stack->top;
        } else {
            stack->stat = RTE_E_NO_DATA;
        }
        result = stack->stat;
    }
    return result;
}
#define RTE_STOP_SEC_CODE
#include "Rte_MemMap.h"

#define RTE_START_SEC_CODE
#include "Rte_MemMap.h"
static inline VAR(Std_ReturnType, AUTOMATIC) Rte_StackPeek(
    P2VAR(RteStackType, AUTOMATIC, RTE_APPL_DATA) stack,
    P2VAR(void, AUTOMATIC, RTE_APPL_DATA) dest,
    VAR(RteStackElementSizeType32, AUTOMATIC) destSize)
{
    Std_ReturnType result;
    if (stack == NULL_PTR) {
        result = RTE_E_LOST_DATA;
    } else {
        if (stack->elementSize != destSize) {
            /*
             * The size of the input element does not match that of the queue element.
             */
            stack->stat |= RTE_E_LOST_DATA;
        } else if (stack->top > 0U) {
            P2CONST(void, AUTOMATIC, RTE_APPL_DATA) src = &(((ByteType8 *)(stack->stackPtr))[(stack->elementSize) * (stack->top - 1U)]);
            (void)qn_memcpy(dest, src, destSize);
            stack->stat = RTE_E_OK;
        } else {
            stack->stat = RTE_E_NO_DATA;
        }
        result = stack->stat;
    }
    return result;
}
#define RTE_STOP_SEC_CODE
#include "Rte_MemMap.h"

extern FUNC(void, RTE_CODE) getPriorRetValue(
    P2VAR(Std_ReturnType, AUTOMATIC, RTE_APPL_DATA) result,
    VAR(Std_ReturnType, AUTOMATIC) status,
    P2CONST(Std_ReturnType, AUTOMATIC, RTE_APPL_DATA) priority,
    VAR(uint32, AUTOMATIC) len);

#ifdef __cplusplus
} /* extern "C" */
#endif /* __cplusplus */

#endif /* RTE_UTIL_H */