/**
 * \file
 * \copyright (c) ESCRYPT GmbH, Bochum, Germany 2019. All rights reserved, also regarding any disposal, exploitation, reproduction, editing, distribution, as well as in the event of applications for industrial property rights.
 */

/* Global QA-C suppression */
/* PRQA S 0776, 0778 EOF #Deviation ID directive_1.1_0776_0778 */
/* PRQA S 0292 EOF #Deviation ID directive_1.1_0292 */
/* PRQA S 0857 EOF #Deviation ID directive_1.1_0857 */
/* PRQA S 2053 EOF #Deviation ID rule_18.8_2053 */
/* PRQA S 3214 EOF #Suppressed because of the ASR memmap concept */
/* PRQA S 2000 EOF #Suppressed for simplicity. Not necessary having else statement is allowed. */
/* PRQA S 5002 EOF #Suppressed as header file names are defined by ASR. */

#ifndef CRYPTO_11_HSM_H
#define CRYPTO_11_HSM_H
/** \req [$Satisfies $SWS_Crypto 00007] File structure */

#include "Crypto_11_Hsm_KeyManagement.h"

#include <Csm_Types.h>
#include <CryIf.h>
#include <Crypto_11_Hsm_Cfg.h>
/** \req [$Satisfies $SWS_Crypto 00009] Inclusion structure */

/* Version check between software and generator */
#include <Crypto_11_Hsm_Version.h>
#if ( (CRYPTO_11_HSM_SW_VERSION_MAIN    != CRYPTO_11_HSM_GEN_VERSION_MAIN)      || \
      (CRYPTO_11_HSM_SW_VERSION_MAJOR   != CRYPTO_11_HSM_GEN_VERSION_MAJOR)     || \
      (CRYPTO_11_HSM_SW_VERSION_MINOR   != CRYPTO_11_HSM_GEN_VERSION_MINOR)     || \
      (CRYPTO_11_HSM_SW_RELEASE_TYPE_I  != CRYPTO_11_HSM_GEN_RELEASE_TYPE_I)    || \
      (CRYPTO_11_HSM_SW_VERSION_PATCH   != CRYPTO_11_HSM_GEN_VERSION_PATCH) )
#error The generator version and software version do not match
#endif

/** \req [$Satisfies $SWS_Crypto 00040] Development error types */
#define CRYPTO_11_HSM_E_UNINIT 0x00u /**< API request called before initialization of Crypto Driver. */
#define CRYPTO_11_HSM_E_INIT_FAILED 0x01u /**< Initialization of Crypto Driver failed */
#define CRYPTO_11_HSM_E_PARAM_POINTER 0x02u /**< API request called with invalid parameter (Null pointer). */
#define CRYPTO_11_HSM_E_PARAM_HANDLE 0x04u /**< API request called with invalid parameter (out of range). */
#define CRYPTO_11_HSM_E_PARAM_VALUE 0x05u /**< API request called with invalid parameter (invalid value). */
/* End of development error types present in requirement SWS_Crypto_00040 */

/** \req [$Satisfies $SWS_Crypto 00194] Runtime error types */
#define CRYPTO_11_HSM_E_RE_SMALL_BUFFER 0x00u /**< Buffer is too small for operation */
#define CRYPTO_11_HSM_E_RE_KEY_NOT_AVAILABLE 0x01u /**< Requested key is not available */
#define CRYPTO_11_HSM_E_RE_KEY_READ_FAIL 0x02u /**< Key cannot be read */
#define CRYPTO_11_HSM_E_RE_ENTROPY_EXHAUSTED 0x03u /**< Entropy is too low */
/* End of runtime error types present in requirement SWS_Crypto_00194 */

/* Specific vendor ID registered with Autosar */
#define CRYPTO_11_HSM_VENDOR_ID_30 30u

#if defined(CRYIF_VENDOR_ID)
#if CRYIF_VENDOR_ID == CRYPTO_11_HSM_VENDOR_ID_30
#define CRYPTO_11_HSM_VENDOR_ID_30_STACK_USED
#endif
#endif

/* PKCS#7 padding mode for ENCRYPT and DECRYPT services as specified by AUTOSAR version 4.4 */
#define CRYPTO_11_HSM_ALGOFAM_PADDING_PKCS7 0x22u

#define CRYPTO_11_HSM_START_SEC_CODE
#include <Crypto_11_Hsm_MemMap.h>

/**
 * \req [$DD 1]
 * \req [$Satisfies $SWS_Crypto 91000][$Satisfies $SWS_Crypto 00019]
 * \req [$Satisfies $ARCH CONFIG_2]
 *
 * \brief Initializes the Crypto Driver.
 *
 * This synchronous function triggers the initialization of the crypto driver and the underlying HSM.
 * The entire initialization is not performed here to avoid long delays at startup, which would not be
 * acceptable, as the entire ECU would be waiting for the initialization of the crypto driver.
 * The callouts to access critical sections and timing functionality are initialized.
 * After this operation, the function checks the current state of the HSM and sets an internal init
 * state variable accordingly, which will be used in the next invocations of Crypto_11_Hsm_MainFunction()
 * to resume the initialization.
 * If the HSM has already been configured before starting AUTOSAR (e.g. during secure boot) then the internal
 * state will be set to just open the necessary CSAI sessions in the following Crypto_11_Hsm_MainFunction call.
 * If the HSM has not been configured, the internal state will be set for a subsequent
 * Crypto_11_Hsm_MainFunction call to do a full HSM configuration (starting from waiting for the HSM to boot)
 * and open CSAI sessions.
 */
FUNC(void, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_Init(void);
#define CRYPTO_11_HSM_SERVICE_ID_INIT 0x00u

#if (CRYPTO_11_HSM_VERSION_INFO_API == STD_ON)
/**
 * \req [$DD 3]
 * \req [$Satisfies $SWS_Crypto 91001][$Satisfies $SWS_Crypto 00047]
 *
 * \brief Returns the version information of this module.
 *
 * \param[in] versionInfo Pointer to where to store the version information of this module.
 */
FUNC(void, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_GetVersionInfo(
    P2VAR(Std_VersionInfoType, AUTOMATIC, AUTOMATIC) versionInfo
);
#define CRYPTO_11_HSM_SERVICE_ID_GET_VERSION_INFO 0x01u
#endif

/**
 * \req [$DD 4]
 * \req [$Satisfies $SWS_Crypto 91003][$Satisfies $SWS_Crypto 00014][$Satisfies $SWS_Crypto 00117][$Satisfies $SWS_Crypto 00017][$Satisfies $SWS_Crypto 00018][$Satisfies $SWS_Crypto 00020][$Satisfies $SWS_Crypto 00118][$Satisfies $SWS_Crypto 00023][$Satisfies $SWS_Crypto 00026][$Satisfies $SWS_Crypto 00027][$Satisfies $SWS_Crypto 00057][$Satisfies $SWS_Crypto 00058][$Satisfies $SWS_Crypto 00059][$Satisfies $SWS_Crypto 00064][$Satisfies $SWS_Crypto 00067][$Satisfies $SWS_Crypto 00070][$Satisfies $SWS_Crypto 00142]
 * \req [$Satisfies $ARCH CONFIG_3][$Satisfies $ARCH CONFIG_4][$Satisfies $ARCH JOBS_1][$Satisfies $ARCH JOBS_3][$Satisfies $ARCH JOBS_5][$Satisfies $ARCH JOBS_6][$Satisfies $ARCH JOBS_10][$Satisfies $ARCH JOBS_11][$Satisfies $ARCH JOBS_13][$Satisfies $ARCH JOBS_14][$Satisfies $ARCH JOBS_15][$Satisfies $ARCH JOBS_16][$Satisfies $ARCH KEYM_31][$Satisfies $ARCH SHE_9][$Satisfies $ARCH SHE_15]
 *
 * \brief Performs the crypto primitive, that is configured in the job parameter.
 *
 * \internal
 * After performing AUTOSAR development mode checks (SWS_Crypto 00057, 00058, 00059, 00067) and checking the crypto
 * driver object state, this function triggers the execution of a job, according to the crypto driver object state.
 * If the crypto driver object has not been configured yet CRYPTO_E_BUSY is returned. If the configuration failed,
 * E_NOT_OK is returned.
 * The implementation follows the activity diagrams described in $ARCH. A sub-function to trigger every service
 * has created. If the job is asynchronous, the job is started and then the function returns, with an
 * appropriate error code. If the job is synchronous, the function waits calls ecy_hsm_Csai_PollHandle() and waits
 * for the completion of the job. Eventual postprocessing operations (for instance for MAC verification) are then
 * performed.
 * A job is scheduled in the HSM using the CSAI session handle corresponding to the Crypto Driver Object assigned to
 * the job. A CSAI job handle is also assigned when starting a job and reset when the job is complete or canceled.
 * \endinternal
 *
 * \note $SWS_Crypto 00118 is implemented as the HSM receives an invalid job handle ecy_hsm_CSAI_INVALID_JOB_HANDLE_VALUE
 * from the Crypto Driver if the job has not started yet and the driver object is in the status DRIVER_OBJECT_IDLE
 *
 * \param[in,out] objectId Holds the identifier of the Crypto Driver Object.
 * \param[in,out] job Pointer to the configuration of the job. Contains structures with job and primitive relevant
 *                    information but also pointer to result buffers.
 * \retval E_OK Request successful
 * \retval E_NOT_OK Request failed or Crypto Driver Object configuration failed
 * \retval CRYPTO_11_HSM_E_BUSY Request Failed, Crypto Driver Object is Busy
 * \retval CRYPTO_11_HSM_E_KEY_NOT_VALID Request failed, the key is not valid
 * \retval CRYPTO_11_HSM_E_KEY_SIZE_MISMATCH Request failed, a key element has the wrong size.
 * \retval CRYPTO_11_HSM_E_QUEUE_FULL Request failed, the queue is full
 * \retval CRYPTO_11_HSM_E_ENTROPY_EXHAUSTION Request failed, the entropy is exhausted
 * \retval CRYPTO_11_HSM_E_SMALL_BUFFER The provided buffer is too small to store the result
 * \retval CRYPTO_11_HSM_E_COUNTER_OVERFLOW The counter is overflowed.
 * \retval CRYPTO_11_HSM_E_JOB_CANCELED The service request failed because the synchronous Job has been canceled.
 * \retval CRYPTO_E_KEY_NOT_AVAILABLE Redirection key element is not present
 * \retval CRYPTO_E_KEY_READ_FAIL Redirection input key element is read protected
 * \retval CRYPTO_E_KEY_WRITE_FAIL Redirection output key element is write protected
 */
FUNC(Std_ReturnType, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_ProcessJob(
    VAR(uint32, AUTOMATIC) objectId,
    P2VAR(Crypto_JobType, AUTOMATIC, CRYPTO_11_HSM_VAR) job
);
#define CRYPTO_11_HSM_SERVICE_ID_PROCESS_JOB 0x03u

/**
 * \req [$DD 5]
 * \req [$Satisfies $SWS_Crypto 00029][$Satisfies $SWS_Crypto 00122][$Satisfies $SWS_Crypto 00143][$Satisfies $SWS_Crypto 00179][$Satisfies $SWS_Crypto 00181][$Satisfies $SWS_Crypto 00144][$Satisfies $SWS_Crypto 00183][$Satisfies $SWS_Crypto 00123][$Satisfies $SWS_Crypto 00124][$Satisfies $SWS_Crypto 00125]
 * \req [$Satisfies $ARCH JOBS_7][$Satisfies $ARCH JOBS_8][$Satisfies $ARCH JOBS_12][$Satisfies $ARCH JOBS_14][$Satisfies $ARCH SHE_15][$Satisfies $ARCH JOBS_18][$Satisfies $ARCH JOBS_17]
 *
 * \brief This interface removes the provided job from the queue and cancels the processing of the job if possible.
 *
 * \internal
 * After performing AUTOSAR development mode checks (SWS_Crypto 00123, 00124 and 00125) and checking the crypto
 * driver object state, this function triggers the execution of a job, according to the crypto driver object state.
 * The implementation follows the activity diagrams described in $ARCH JOBS_7.
 * There are two possible cases: a job is interrupted while executing (state PROCESSING_STANDARD and
 * PROCESSING_LAST_BLOCK) or waiting for the next block to be processed (state WAITING_FOR_INPUT for streaming jobs).
 * In the first case ecy_hsm_Csai_CancelOperation() is called. The job cancellation will then be performed
 * asynchronously. Crypto_11_Hsm_MainFunction() will check the job cancellation status. E_OK or E_NOT_OK is
 * returned, according to whether the cancellation request was successful or not.
 * In the second case ecy_hsm_Csai_CancelJob() is invoked. As the function runs synchronously and quickly,
 * the job is immediately cancelled. No asynchronous calls are need in the main function. CryIf is notified through
 * CryIf_CallbackNotification(), with the CRYPTO_E_JOB_CANCELED error code and E_OK is returned.
 * \endinternal
 *
 * \param [in] objectId Holds the identifier of the Crypto Driver Object.
 * @cond
 * \param [in,out] job Pointer to the configuration of the job. Contains structures with job and primitive relevant information. If defined(CRYPTO_11_HSM_VECTOR_STACK_USED)
 * @endcond
 * \param [in,out] jobInfo Pointer to the configuration of the job. Contains structures with job and primitive relevant information. If not defined(CRYPTO_11_HSM_VECTOR_STACK_USED)
 * \retval E_OK Request successful, job has been removed.
 * \retval E_NOT_OK Request Failed, job couldn't be removed.
 * \retval CRYPTO_11_HSM_E_JOB_CANCELED The job has been cancelled but is still processed. No results will be returned to the application.
 */
FUNC(Std_ReturnType, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_CancelJob(
    VAR(uint32, AUTOMATIC) objectId,
#if defined(CRYPTO_11_HSM_VENDOR_ID_30_STACK_USED)
    P2VAR(Crypto_JobType, AUTOMATIC, CRYPTO_11_HSM_VAR) job
#else
    CONSTP2CONST(Crypto_JobInfoType, AUTOMATIC, CRYPTO_11_HSM_CONST) jobInfo
#endif
);
#define CRYPTO_11_HSM_SERVICE_ID_CANCEL_JOB 0x0eu

/**
 * \req [$DD 2]
 * \req [$Satisfies $SWS_Crypto 91012][$Satisfies $SWS_Crypto 00025][$Satisfies $SWS_Crypto 00119][$Satisfies $SWS_Crypto 00027][$Satisfies $SWS_Crypto 00028]
 * \req [$Satisfies $ARCH CONFIG_1][$Satisfies $ARCH CONFIG_2][$Satisfies $ARCH CONFIG_5][$Satisfies $ARCH CONFIG_6][$Satisfies $ARCH JOBS_1][$Satisfies $ARCH JOBS_4][$Satisfies $ARCH JOBS_9][$Satisfies $ARCH JOBS_14][$Satisfies $ARCH JOBS_16][$Satisfies $ARCH KEYM_31][$Satisfies $ARCH SHE_9][$Satisfies $ARCH SHE_15]
 *
 * \brief It completes Crypto Driver configuration and processes active jobs.
 *
 * If the HSM is not pre-configured, it waits for the HSM to boot, it configures the HSM clock, releases the HSM,
 * waits for the HSM to be in application mode and configures the HSM. All these operations are performed
 * asynchronously. The user shall invoke this function periodically (as specified in the AUTOSAR standard). This
 * advances the Crypto Driver configuration. The HSM sessions corresponding to the Crypto Driver Objects are
 * then opened. Until this moment, any incoming job will be rejected as the Crypto Driver is still busy.
 *
 * After the configuration is completed, this functions checks the status of all Crypto Driver Objects and advances
 * the status of the job under processing.
 *
 * If the configuration finishes with an error, the crypto driver state is set to DRIVER_CONFIG_STATE_FAIL.
 * No further job will be processed. E_NOT_OK will be returned to Crypto_11_Hsm_ProcessJob().
 *
 * \internal
 * The crypto driver configuration goes through the following states:
 *
 * - State DRIVER_CONFIG_STATE_UNINIT: Crypto_11_Hsm_Init() has not been called yet, configuration cannot start
 * - State DRIVER_CONFIG_STATE_WAITING_FOR_BOOTED: wait for HSM to boot and configure the clock and release the HSM
 * - State DRIVER_CONFIG_STATE_WAITING_FOR_APP_MODE: wait until the HSM is in application mode and configure the HSM
 * - State DRIVER_CONFIG_STATE_OPEN_SESSIONS: open HSM sessions corresponding to the crypto driver objects,
 * maximum CRYPTO_11_HSM_CONFIG_NUM_SESSIONS_OPEN_PER_CALL sessions per function invocation
 * - State DRIVER_CONFIG_STATE_FAIL: configuration has finished with an error.
 * - State DRIVER_CONFIG_STATE_SUCCESS: configuration is completed successfully.
 *
 * States DRIVER_CONFIG_STATE_WAITING_FOR_BOOTED and DRIVER_CONFIG_STATE_WAITING_FOR_APP_MODE are monitored for timeout.
 * If timeout occurs, state DRIVER_CONFIG_STATE_FAIL is latched, therefore Crypto_11_Hsm_GetDriverStatus() will return E_NOT_OK.
 * The timeout period starts when either of these two states is first entered.
 * The timeout period is checked at the end of subsequent Crypto_11_Hsm_MainFunction calls for the same state.
 * The timeout period is CRYPTO_11_HSM_INIT_STATE_TIMEOUT_MSEC
 * which is derived from the configuration HsmInitStateTimeoutInMilliseconds in Hsm_ParamDef/Crypto_ecuc ARXML files.
 *
 * \note The states DRIVER_CONFIG_STATE_WAITING_FOR_BOOTED and DRIVER_CONFIG_STATE_WAITING_FOR_APP_MODE exist only when
 * the HSM has not been pre-configured.
 *
 * After the crypto driver has been configured successfully, this function follows the state diagram in
 * $ARCH 10 in order to process asynchronous jobs (synchronous jobs are ignored), with the states:
 *
 * - DRIVER_OBJECT_STATE_IDLE (Idle),
 * - DRIVER_OBJECT_STATE_PROCESSING_STANDARD (Processing standard job)
 * - DRIVER_OBJECT_STATE_WAITING_FOR_INPUT (Waiting for input),
 * - DRIVER_OBJECT_STATE_PROCESSING_LAST_BLOCK (Processing last block)
 * - DRIVER_OBJECT_STATE_PROCESSING_CANCELLATION (Processing cancellation)
 *
 * \endinternal
 *
 * \todo Investigating value for NUM_SESSION_OPEN_PER_CALL
 */
FUNC(void, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_MainFunction(void);
#define CRYPTO_11_HSM_SERVICE_ID_MAIN_FUNCTION 0x0cu

/**
 * \req [$DD 109]
 * \req [$Satisfies $SWS_Crypto 00123][$Satisfies $SWS_Crypto 00168][$Satisfies $SWS_Crypto 00156][$Satisfies $SWS_Crypto 00097][$Satisfies $SWS_Crypto 00149][$Satisfies $SWS_Crypto 00085][$Satisfies $SWS_Crypto 00161][$Satisfies $SWS_Crypto 00075]
 * \req [$Satisfies $SWS_Crypto 00103][$Satisfies $SWS_Crypto 00111][$Satisfies $SWS_Crypto 00094][$Satisfies $SWS_Crypto 00082][$Satisfies $SWS_Crypto 00196][$Satisfies $SWS_Crypto 00057][$Satisfies $SWS_Crypto 00128]
 * \brief Return the crypto driver initialization state, i.e. whether or not Crypto_11_Hsm_Init has been called.
 * \note The crypto driver is not ready for use until it is fully configured - see Crypto_11_Hsm_GetDriverStatus.
 *
 * \retval FALSE Crypto_11_Hsm_Init() has not yet been called.
 * \retval TRUE  Crypto_11_Hsm_Init() has been called.
 */
FUNC(boolean, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_IsInitialized(void);

/**
 * \req [$DD 110]
 * \req [$Satisfies $ARCH CONFIG_3][$Satisfies $ARCH CONFIG_4]
 * \brief Return the crypto driver state.
 *
 * \note The crypto driver configuration is performed in stages by Crypto_11_Hsm_Init and Crypto_11_Hsm_MainFunction.
 *
 * \retval E_OK - Crypto Driver successfully completed configuration.
 * \retval E_NOT_OK - Crypto Driver configuration has failed or HSM got into unrecoverable state.
 * \retval CRYPTO_E_BUSY - Crypto Driver configuration is not yet finished.
 */
FUNC(Std_ReturnType, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_GetDriverStatus(void);

/**
 * \req [$DD 106]
 * \req [$Satisfies $ARCH CONFIG_7]
 *
 * \brief Shuts down the Crypto Driver.
 *
 * This synchronous function triggers the shutdown of the crypto driver and the underlying HSM sessions.
 * All open HSM sessions will be closed which automatically unloads all active HSM keys from memory.
 */
FUNC(void, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_DeInit(void);

#define CRYPTO_11_HSM_STOP_SEC_CODE
#include <Crypto_11_Hsm_MemMap.h>

#endif
