/**
 * \file
 * \copyright (c) ESCRYPT GmbH, Bochum, Germany 2019. All rights reserved, also regarding any disposal, exploitation, reproduction, editing, distribution, as well as in the event of applications for industrial property rights.
 */

/* Global QA-C suppression */
/* PRQA S 0776, 0778 EOF #Deviation ID directive_1.1_0776_0778 */
/* PRQA S 0292 EOF #Deviation ID directive_1.1_0292 */
/* PRQA S 0857 EOF #Deviation ID directive_1.1_0857 */
/* PRQA S 2053 EOF #Deviation ID rule_18.8_2053 */
/* PRQA S 3214 EOF #Suppressed because of the ASR memmap concept */
/* PRQA S 2000 EOF #Suppressed for simplicity. Not necessary having else statement is allowed. */
/* PRQA S 5002 EOF #Suppressed as header file names are defined by ASR. */

#ifndef CRYPTO_11_HSM_SERVICES_H
#define CRYPTO_11_HSM_SERVICES_H

#include <Std_Types.h>
#include <Csm_Types.h>
#include <Crypto_11_Hsm_Cfg.h>
#include <Crypto_11_Hsm_Timer.h>

#include <ecy_hsm_csai.h>
#include <ecy_hsm_csai_mac.h>
#include <ecy_hsm_csai_sign.h>
#include <ecy_hsm_csai_hash.h>
#include <ecy_hsm_csai_crypt.h>
#include <ecy_hsm_she.h>

#define CRYPTO_11_HSM_SERVICE_STD_COUNT 12u

#if (CRYPTO_11_HSM_CONFIG_ENABLE_EXTENSION == STD_ON)
#define CRYPTO_11_HSM_SERVICE_EXT_COUNT 9u
#else
#define CRYPTO_11_HSM_SERVICE_EXT_COUNT 0u
#endif

#if (CRYPTO_11_HSM_CONFIG_ENABLE_MDK == STD_ON)
#define CRYPTO_11_HSM_SERVICE_MDK_COUNT 23u
#else
#define CRYPTO_11_HSM_SERVICE_MDK_COUNT 0u
#endif

/* Total number of crypto services defined by CSM */
#define CRYPTO_11_HSM_SERVICE_COUNT (CRYPTO_11_HSM_SERVICE_STD_COUNT + CRYPTO_11_HSM_SERVICE_EXT_COUNT + CRYPTO_11_HSM_SERVICE_MDK_COUNT)

/* IV length for encryption/decryption in CBC mode */
#define CRYPTO_11_HSM_IV_LENGTH 16u

typedef struct
{
    uint32 resultFlag;
} Crypto_11_Hsm_ServiceData_MacVerify;

typedef struct
{
    uint32 resultFlag;
} Crypto_11_Hsm_ServiceData_SignVerify;

typedef struct
{
    uint32 outputLengthFinishUpdate;
    uint32 outputLengthFinish;
#if (((CRYPTO_11_HSM_CONFIG_HAS_DECRYPT == STD_ON) || (CRYPTO_11_HSM_CONFIG_HAS_ENCRYPT == STD_ON)) && \
     ((CRYPTO_11_HSM_CONFIG_HAS_ALGOMODE_CBC == STD_ON) || (CRYPTO_11_HSM_CONFIG_HAS_ALGOMODE_CTR == STD_ON)) && \
     ((CRYPTO_11_HSM_CONFIG_HAS_ALGOFAM_AES == STD_ON)))
    uint8 iv[CRYPTO_11_HSM_IV_LENGTH];
#else
    uint8* iv;
#endif
} Crypto_11_Hsm_ServiceData_EncryptDecrypt;

typedef struct
{
    uint32 value1;
    uint32 value2;
} Crypto_11_Hsm_ServiceData_Generic;

/* PRQA S 0750,3630 3 # union used to save memory, a job is based on exactly one service and only its corresponding data is accessed during its processing */
typedef union
{
    Crypto_11_Hsm_ServiceData_MacVerify  macVerify;
    Crypto_11_Hsm_ServiceData_SignVerify signVerify;
    Crypto_11_Hsm_ServiceData_EncryptDecrypt encryptDecrypt;
    Crypto_11_Hsm_ServiceData_Generic generic;
} Crypto_11_Hsm_ServiceData;

/**
 * \brief Pointer to a crypto service function
 */
/* PRQA S 5004 1 #Function pointer is used safely to provide better architecture separation in the code */
typedef ecy_hsm_Csai_ErrorT (*Crypto_11_Hsm_Service_Func_PtrType)(
    const ecy_hsm_Csai_SessionHandleT hSession,
    const ecy_hsm_Csai_KeyHandleT hKey,
    ecy_hsm_Csai_JobHandleT *const phJob,
    const ecy_hsm_Csai_JobPriorityT priority,
    const Crypto_JobType *const job,
    Crypto_11_Hsm_ServiceData *const data,
    const boolean finishAfterUpdate);

/**
 * \brief Pointer to a post processing crypto service function
 */
/* PRQA S 5004 1 #Function pointer is used safely to provide better architecture separation in the code */
typedef void (*Crypto_11_Hsm_PostProcessing_Service_Func_PtrType)(
    ecy_hsm_Csai_ErrorT *const pError,
    const Crypto_JobType *const job,
    const Crypto_11_Hsm_ServiceData *const data);

/**
 * \brief Pointer to a SHE function
 */
/* PRQA S 5004 1 #Function pointer is used safely to provide better architecture separation in the code */
typedef ecy_hsm_Csai_ErrorT (*Crypto_11_Hsm_SHE_Service_Func_PtrType)(
    const SHE_KeyIdT keyId,
    const Crypto_JobType *const job,
    Crypto_11_Hsm_ServiceData *const data);

/**
 * \brief Pointer to a post processing SHE function
 */
/* PRQA S 5004 1 #Function pointer is used safely to provide better architecture separation in the code */
typedef void (*Crypto_11_Hsm_SHE_PostProcessing_Service_Func_PtrType)(
    const ecy_hsm_Csai_ErrorT *const pError,
    const Crypto_JobType *const job,
    const Crypto_11_Hsm_ServiceData *const data);

/**
 * \brief Crypto service dispatch table definition
 */
typedef struct
{
  Crypto_11_Hsm_Service_Func_PtrType Crypto_11_Hsm_Service_Func;
  Crypto_11_Hsm_PostProcessing_Service_Func_PtrType Crypto_11_Hsm_PostProcessing_Service_Func;
  Crypto_11_Hsm_SHE_Service_Func_PtrType Crypto_11_Hsm_SHE_Service_Func;
  Crypto_11_Hsm_SHE_PostProcessing_Service_Func_PtrType Crypto_11_Hsm_SHE_PostProcessing_Service_Func;
  boolean ServiceNeedsKey;
} Crypto_11_Hsm_Service_Dispatch;

#define CRYPTO_11_HSM_START_SEC_CODE
#include <Crypto_11_Hsm_MemMap.h>

FUNC(ecy_hsm_Csai_ErrorT, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_ProcessJob_Dummy(
    CONST(ecy_hsm_Csai_SessionHandleT, AUTOMATIC) hSession,
    CONST(ecy_hsm_Csai_KeyHandleT, AUTOMATIC) hKey,
    CONSTP2VAR(ecy_hsm_Csai_JobHandleT, AUTOMATIC, CRYPTO_11_HSM_VAR) phJob,
    CONST(ecy_hsm_Csai_JobPriorityT, AUTOMATIC) priority,
    CONSTP2CONST(Crypto_JobType, AUTOMATIC, CRYPTO_11_HSM_CONST) job,
    CONSTP2VAR(Crypto_11_Hsm_ServiceData, AUTOMATIC, CRYPTO_11_HSM_VAR) data,
    CONST(boolean, AUTOMATIC) finishAfterUpdate
);

FUNC(void, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_PostProcessing_Dummy(
    CONSTP2VAR(ecy_hsm_Csai_ErrorT, AUTOMATIC, AUTOMATIC) pError,
    CONSTP2CONST(Crypto_JobType, AUTOMATIC, CRYPTO_11_HSM_CONST) job,
    CONSTP2CONST(Crypto_11_Hsm_ServiceData, AUTOMATIC, CRYPTO_11_HSM_CONST) data
);

#if (CRYPTO_11_HSM_CONFIG_HAS_HASH == STD_ON)
/**
 * \req [$DD 6]
 * \req [$Satisfies $SWS_Crypto 00065][$Satisfies $SWS_Crypto 00071][$Satisfies $SWS_Crypto 00073][$Satisfies $SWS_Crypto 00134][$Satisfies $SWS_Crypto 00135][$Satisfies $SWS_Crypto 91003]
 * \req [$Satisfies $ARCH HASH_1][$Satisfies $ARCH HASH_2][$Satisfies $ARCH JOBS_19]
 *
 * \brief Performs the crypto primitive, that is configured in the job parameter.
 *
 * \internal
 * After performing AUTOSAR development mode checks (SWS_Crypto 00070, 00142) this function maps the AUTOSAR
 * algorithm and job priority to the ones available in the CSAI. Then calls the appropriate primitive
 * (start, update, finish or fast call) according to the job mode (flags start, update and finish and their
 * combination).
 *
 * \param[in] hSession HSM session corresponding to the crypto driver object
 * \param[in] hKey Set to NULL_PTR for this service
 * \param[in,out] phJob Pointer to the HSM job executing the service
 * \param[in] priority Job priority
 * \param[in] job Crypto Driver Object job under processing
 * \param[in] data Input data to the service
 * \param[in,out] finishAfterUpdate TRUE when this function is called from Crypto_11_Hsm_MainFunction() to call
 *                the csai finish hash primitive in processing last block crypto driver object state
 * \retval ecy_hsm_CSAI_SUCCESS Request successful
 * \retval ecy_hsm_CSAI_ERR_INVALID_PARAM DET error check failed or CSAI parameter invalid
 * \retval Other error CSAI hash operation return code
 */
FUNC(ecy_hsm_Csai_ErrorT, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_ProcessJob_Hash(
    CONST(ecy_hsm_Csai_SessionHandleT, AUTOMATIC) hSession,
    CONST(ecy_hsm_Csai_KeyHandleT, AUTOMATIC) hKey,
    CONSTP2VAR(ecy_hsm_Csai_JobHandleT, AUTOMATIC, CRYPTO_11_HSM_VAR) phJob,
    CONST(ecy_hsm_Csai_JobPriorityT, AUTOMATIC) priority,
    CONSTP2CONST(Crypto_JobType, AUTOMATIC, CRYPTO_11_HSM_CONST) job,
    CONSTP2VAR(Crypto_11_Hsm_ServiceData, AUTOMATIC, CRYPTO_11_HSM_VAR) data,
    CONST(boolean, AUTOMATIC) finishAfterUpdate
);
#else
#define Crypto_11_Hsm_ProcessJob_Hash   Crypto_11_Hsm_ProcessJob_Dummy
#endif

#if (CRYPTO_11_HSM_CONFIG_HAS_MACVERIFY == STD_ON)
/**
 * \req [$DD 7]
 * \req [$Satisfies $SWS_Crypto 00072][$Satisfies $SWS_Crypto 00071][$Satisfies $SWS_Crypto 00073][$Satisfies $SWS_Crypto 00134][$Satisfies $SWS_Crypto 91003]
 * \req [$Satisfies $ARCH MAC_3][$Satisfies $ARCH MAC_4][$Satisfies $ARCH JOBS_19]
 *
 * \brief Performs the crypto primitive, that is configured in the job parameter.
 *
 * \internal
 * After performing AUTOSAR development mode checks (SWS_Crypto 00070, 00142) this function maps the AUTOSAR
 * algorithm and job priority to the ones available in the CSAI. Then calls the appropriate primitive
 * (start, update, finish or fast call) according to the job mode (flags start, update and finish and their
 * combination).
 *
 * \param[in] hSession HSM session corresponding to the crypto driver object
 * \param[in] hKey Key handle or key ID for the HSM job executing the service
 * \param[in,out] phJob Pointer to the HSM job executing the service
 * \param[in] priority Job priority
 * \param[in] job Crypto Driver Object job under processing
 * \param[in] data Input data to the service
 * \param[in,out] finishAfterUpdate TRUE when this function is called from Crypto_11_Hsm_MainFunction() to call
 *                the csai finish mac verify primitive in processing last block crypto driver object state
 * \retval ecy_hsm_CSAI_SUCCESS Request successful
 * \retval ecy_hsm_CSAI_ERR_INVALID_PARAM DET error check failed or CSAI parameter invalid
 * \retval Other error CSAI mac verify operation return code
 */
FUNC(ecy_hsm_Csai_ErrorT, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_ProcessJob_MacVerify(
    CONST(ecy_hsm_Csai_SessionHandleT, AUTOMATIC) hSession,
    CONST(ecy_hsm_Csai_KeyHandleT, AUTOMATIC) hKey,
    CONSTP2VAR(ecy_hsm_Csai_JobHandleT, AUTOMATIC, CRYPTO_11_HSM_VAR) phJob,
    CONST(ecy_hsm_Csai_JobPriorityT, AUTOMATIC) priority,
    CONSTP2CONST(Crypto_JobType, AUTOMATIC, CRYPTO_11_HSM_CONST) job,
    CONSTP2VAR(Crypto_11_Hsm_ServiceData, AUTOMATIC, CRYPTO_11_HSM_VAR) data,
    CONST(boolean, AUTOMATIC) finishAfterUpdate
);

/**
 * \req [$DD 8]
 * \req [$Satisfies $SWS_Crypto 91003]
 * \req [$Satisfies $ARCH MAC_3][$Satisfies $ARCH MAC_4]
 */
FUNC(void, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_PostProcessing_MacVerify(
    CONSTP2VAR(ecy_hsm_Csai_ErrorT, AUTOMATIC, AUTOMATIC) pError,
    CONSTP2CONST(Crypto_JobType, AUTOMATIC, CRYPTO_11_HSM_CONST) job,
    CONSTP2CONST(Crypto_11_Hsm_ServiceData, AUTOMATIC, CRYPTO_11_HSM_CONST) data
);

#else
#define Crypto_11_Hsm_ProcessJob_MacVerify          Crypto_11_Hsm_ProcessJob_Dummy
#define Crypto_11_Hsm_PostProcessing_MacVerify      Crypto_11_Hsm_PostProcessing_Dummy
#endif


#if (CRYPTO_11_HSM_CONFIG_HAS_MACGENERATE == STD_ON)
/**
 * \req [$DD 10]
 * \req [$Satisfies $SWS_Crypto 00065][$Satisfies $SWS_Crypto 00072][$Satisfies $SWS_Crypto 00071][$Satisfies $SWS_Crypto 00073][$Satisfies $SWS_Crypto 00134][$Satisfies $SWS_Crypto 00135][$Satisfies $SWS_Crypto 91003]
 * \req [$Satisfies $ARCH MAC_1][$Satisfies $ARCH MAC_2][$Satisfies $ARCH JOBS_19]
 *
 * \brief Performs the crypto primitive, that is configured in the job parameter.
 *
 * \internal
 * After performing AUTOSAR development mode checks (SWS_Crypto 00070, 00142) this function maps the AUTOSAR
 * algorithm and job priority to the ones available in the CSAI. Then calls the appropriate primitive
 * (start, update, finish or fast call) according to the job mode (flags start, update and finish and their
 * combination).
 *
 * \param[in] hSession HSM session corresponding to the crypto driver object
 * \param[in] hKey Key handle or key ID for the HSM job executing the service
 * \param[in,out] phJob Pointer to the HSM job executing the service
 * \param[in] priority Job priority
 * \param[in] job Crypto Driver Object job under processing
 * \param[in] data Input data to the service
 * \param[in,out] finishAfterUpdate TRUE when this function is called from Crypto_11_Hsm_MainFunction() to call
 *                the csai finish mac generate primitive in processing last block crypto driver object state
 * \retval ecy_hsm_CSAI_SUCCESS Request successful
 * \retval ecy_hsm_CSAI_ERR_INVALID_PARAM DET error check failed or CSAI parameter invalid
 * \retval Other error CSAI mac generate operation return code
 */
FUNC(ecy_hsm_Csai_ErrorT, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_ProcessJob_MacGenerate(
    CONST(ecy_hsm_Csai_SessionHandleT, AUTOMATIC) hSession,
    CONST(ecy_hsm_Csai_KeyHandleT, AUTOMATIC) hKey,
    CONSTP2VAR(ecy_hsm_Csai_JobHandleT, AUTOMATIC, CRYPTO_11_HSM_VAR) phJob,
    CONST(ecy_hsm_Csai_JobPriorityT, AUTOMATIC) priority,
    CONSTP2CONST(Crypto_JobType, AUTOMATIC, CRYPTO_11_HSM_CONST) job,
    CONSTP2VAR(Crypto_11_Hsm_ServiceData, AUTOMATIC, CRYPTO_11_HSM_VAR) data,
    CONST(boolean, AUTOMATIC) finishAfterUpdate
);
#else
#define Crypto_11_Hsm_ProcessJob_MacGenerate        Crypto_11_Hsm_ProcessJob_Dummy
#endif

#if (CRYPTO_11_HSM_CONFIG_HAS_ENCRYPT == STD_ON)
/**
 * \req [$DD 22]
 * \req [$Satisfies $SWS_Crypto 00072][$Satisfies $SWS_Crypto 00071][$Satisfies $SWS_Crypto 00073][$Satisfies $SWS_Crypto 00134][$Satisfies $SWS_Crypto 00135][$Satisfies $SWS_Crypto 91003]
 * \req [$Satisfies $ARCH ENC_1] [$Satisfies $ARCH ENC_2][$Satisfies $ARCH JOBS_19]
 *
 * \brief Performs the service "encrypt" with the crypto primitive, that is configured in the job parameter.
 *
 * \internal
 * After performing AUTOSAR development mode checks (SWS_Crypto 00070, 00142) this function maps the AUTOSAR
 * algorithm and job priority to the ones available in the CSAI. Then calls the appropriate primitive
 * (start, update, finish or fast call) according to the job mode (flags start, update and finish and their
 * combination).
 *
 * \param[in] hSession HSM session corresponding to the crypto driver object
 * \param[in] hKey Key handle or key ID for the HSM job executing the service
 * \param[in,out] phJob Pointer to the HSM job executing the service
 * \param[in] priority Job priority
 * \param[in] job Crypto Driver Object job under processing
 * \param[in] data Input data to the service
 * \param[in,out] finishAfterUpdate TRUE when this function is called from Crypto_11_Hsm_MainFunction() to call
 *                the csai finish aes primitive in processing last block crypto driver object state
 * \retval ecy_hsm_CSAI_SUCCESS Request successful
 * \retval ecy_hsm_CSAI_ERR_INVALID_PARAM DET error check failed or CSAI parameter invalid
 */
FUNC(ecy_hsm_Csai_ErrorT, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_ProcessJob_Encrypt(
    CONST(ecy_hsm_Csai_SessionHandleT, AUTOMATIC) hSession,
    CONST(ecy_hsm_Csai_KeyHandleT, AUTOMATIC) hKey,
    CONSTP2VAR(ecy_hsm_Csai_JobHandleT, AUTOMATIC, CRYPTO_11_HSM_VAR) phJob,
    CONST(ecy_hsm_Csai_JobPriorityT, AUTOMATIC) priority,
    CONSTP2CONST(Crypto_JobType, AUTOMATIC, CRYPTO_11_HSM_CONST) job,
    CONSTP2VAR(Crypto_11_Hsm_ServiceData, AUTOMATIC, CRYPTO_11_HSM_VAR) data,
    CONST(boolean, AUTOMATIC) finishAfterUpdate
);
#else
#define Crypto_11_Hsm_ProcessJob_Encrypt            Crypto_11_Hsm_ProcessJob_Dummy
#endif

#if (CRYPTO_11_HSM_CONFIG_HAS_DECRYPT == STD_ON)
/**
 * \req [$DD 23]
 * \req [$Satisfies $SWS_Crypto 00072][$Satisfies $SWS_Crypto 00071][$Satisfies $SWS_Crypto 00073][$Satisfies $SWS_Crypto 00134][$Satisfies $SWS_Crypto 00135][$Satisfies $SWS_Crypto 91003]
 * \req [$Satisfies $ARCH DEC_1] [$Satisfies $ARCH DEC_2][$Satisfies $ARCH JOBS_19]
 *
 * \brief Performs the service "decrypt" with the crypto primitive, that is configured in the job parameter.
 *
 * \internal
 * After performing AUTOSAR development mode checks (SWS_Crypto 00070, 00142) this function maps the AUTOSAR
 * algorithm and job priority to the ones available in the CSAI. Then calls the appropriate primitive
 * (start, update, finish or fast call) according to the job mode (flags start, update and finish and their
 * combination).
 *
 * \param[in] hSession HSM session corresponding to the crypto driver object
 * \param[in] hKey Key handle or key ID for the HSM job executing the service
 * \param[in,out] phJob Pointer to the HSM job executing the service
 * \param[in] priority Job priority
 * \param[in] job Crypto Driver Object job under processing
 * \param[in] data Input data to the service
 * \param[in,out] finishAfterUpdate TRUE when this function is called from Crypto_11_Hsm_MainFunction() to call
 *                the csai finish aes primitive in processing last block crypto driver object state
 * \retval ecy_hsm_CSAI_SUCCESS Request successful
 * \retval ecy_hsm_CSAI_ERR_INVALID_PARAM DET error check failed or CSAI parameter invalid
 */
FUNC(ecy_hsm_Csai_ErrorT, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_ProcessJob_Decrypt(
    CONST(ecy_hsm_Csai_SessionHandleT, AUTOMATIC) hSession,
    CONST(ecy_hsm_Csai_KeyHandleT, AUTOMATIC) hKey,
    CONSTP2VAR(ecy_hsm_Csai_JobHandleT, AUTOMATIC, CRYPTO_11_HSM_VAR) phJob,
    CONST(ecy_hsm_Csai_JobPriorityT, AUTOMATIC) priority,
    CONSTP2CONST(Crypto_JobType, AUTOMATIC, CRYPTO_11_HSM_CONST) job,
    CONSTP2VAR(Crypto_11_Hsm_ServiceData, AUTOMATIC, CRYPTO_11_HSM_VAR) data,
    CONST(boolean, AUTOMATIC) finishAfterUpdate
);
#else
#define Crypto_11_Hsm_ProcessJob_Decrypt            Crypto_11_Hsm_ProcessJob_Dummy
#endif

#if (CRYPTO_11_HSM_CONFIG_HAS_ENCRYPT == STD_ON) || (CRYPTO_11_HSM_CONFIG_HAS_DECRYPT == STD_ON)
/**
 * \req [$DD 24]
 * \req [$Satisfies $ARCH ENC_1][$Satisfies $ARCH DEC_1]
 */
FUNC(void, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_PostProcessing_EncryptDecrypt(
    CONSTP2VAR(ecy_hsm_Csai_ErrorT, AUTOMATIC, AUTOMATIC) pError,
    CONSTP2CONST(Crypto_JobType, AUTOMATIC, CRYPTO_11_HSM_CONST) job,
    CONSTP2CONST(Crypto_11_Hsm_ServiceData, AUTOMATIC, CRYPTO_11_HSM_CONST) data
);
#else
#define Crypto_11_Hsm_PostProcessing_EncryptDecrypt Crypto_11_Hsm_PostProcessing_Dummy
#endif

#if (CRYPTO_11_HSM_CONFIG_HAS_SIGNATUREVERIFY == STD_ON)
/**
 * \req [$DD 15]
 * \req [$Satisfies $SWS_Crypto 00072][$Satisfies $SWS_Crypto 00071][$Satisfies $SWS_Crypto 00073][$Satisfies $SWS_Crypto 00134][$Satisfies $SWS_Crypto 91003]
 * \req [$Satisfies $ARCH SIGN_3][$Satisfies $ARCH SIGN_4][$Satisfies $ARCH JOBS_19]
 *
 * \brief Performs the crypto primitive, that is configured in the job parameter.
 *
 * \internal
 * After performing AUTOSAR development mode checks (SWS_Crypto 00070, 00142) this function maps the AUTOSAR
 * algorithm and job priority to the ones available in the CSAI. Then calls the appropriate primitive
 * (start, update, finish or fast call) according to the job mode (flags start, update and finish and their
 * combination).
 *
 * \param[in] hSession HSM session corresponding to the crypto driver object
 * \param[in] hKey Key handle or key ID for the HSM job executing the service
 * \param[in,out] phJob Pointer to the HSM job executing the service
 * \param[in] priority Job priority
 * \param[in] job Crypto Driver Object job under processing
 * \param[in] data Input data to the service
 * \param[in,out] finishAfterUpdate TRUE when this function is called from Crypto_11_Hsm_MainFunction() to call
 *                the csai finish mac verify primitive in processing last block crypto driver object state
 * \retval ecy_hsm_CSAI_SUCCESS Request successful
 * \retval ecy_hsm_CSAI_ERR_INVALID_PARAM DET error check failed or CSAI parameter invalid
 * \retval Other error CSAI mac verify operation return code
 */
FUNC(ecy_hsm_Csai_ErrorT, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_ProcessJob_SigVerify(
    CONST(ecy_hsm_Csai_SessionHandleT, AUTOMATIC) hSession,
    CONST(ecy_hsm_Csai_KeyHandleT, AUTOMATIC) hKey,
    CONSTP2VAR(ecy_hsm_Csai_JobHandleT, AUTOMATIC, CRYPTO_11_HSM_VAR) phJob,
    CONST(ecy_hsm_Csai_JobPriorityT, AUTOMATIC) priority,
    CONSTP2CONST(Crypto_JobType, AUTOMATIC, CRYPTO_11_HSM_CONST) job,
    CONSTP2VAR(Crypto_11_Hsm_ServiceData, AUTOMATIC, CRYPTO_11_HSM_VAR) data,
    CONST(boolean, AUTOMATIC) finishAfterUpdate
);

/**
 * \req [$DD 16]
 * \req [$Satisfies $SWS_Crypto 91003]
 * \req [$Satisfies $ARCH SIGN_3][$Satisfies $ARCH SIGN_4]
 */
FUNC(void, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_PostProcessing_SigVerify(
    CONSTP2VAR(ecy_hsm_Csai_ErrorT, AUTOMATIC, AUTOMATIC) pError,
    CONSTP2CONST(Crypto_JobType, AUTOMATIC, CRYPTO_11_HSM_CONST) job,
    CONSTP2CONST(Crypto_11_Hsm_ServiceData, AUTOMATIC, CRYPTO_11_HSM_CONST) data
);
#else
#define Crypto_11_Hsm_ProcessJob_SigVerify          Crypto_11_Hsm_ProcessJob_Dummy
#define Crypto_11_Hsm_PostProcessing_SigVerify      Crypto_11_Hsm_PostProcessing_Dummy
#endif


#if (CRYPTO_11_HSM_CONFIG_HAS_SIGNATUREGENERATE == STD_ON)
/**
 * \req [$DD 17]
 * \req [$Satisfies $SWS_Crypto 00072][$Satisfies $SWS_Crypto 00071][$Satisfies $SWS_Crypto 00073][$Satisfies $SWS_Crypto 00134][$Satisfies $SWS_Crypto 00135][$Satisfies $SWS_Crypto 91003]
 * \req [$Satisfies $ARCH SIGN_1][$Satisfies $ARCH SIGN_2][$Satisfies $ARCH JOBS_19]
 *
 * \brief Performs the crypto primitive, that is configured in the job parameter.
 *
 * \internal
 * After performing AUTOSAR development mode checks (SWS_Crypto 00070, 00142) this function maps the AUTOSAR
 * algorithm and job priority to the ones available in the CSAI. Then calls the appropriate primitive
 * (start, update, finish or fast call) according to the job mode (flags start, update and finish and their
 * combination).
 *
 * \param[in] hSession HSM session corresponding to the crypto driver object
 * \param[in] hKey Key handle or key ID for the HSM job executing the service
 * \param[in,out] phJob Pointer to the HSM job executing the service
 * \param[in] priority Job priority
 * \param[in] job Crypto Driver Object job under processing
 * \param[in] data Input data to the service
 * \param[in,out] finishAfterUpdate TRUE when this function is called from Crypto_11_Hsm_MainFunction() to call
 *                the csai finish mac generate primitive in processing last block crypto driver object state
 * \retval ecy_hsm_CSAI_SUCCESS Request successful
 * \retval ecy_hsm_CSAI_ERR_INVALID_PARAM DET error check failed or CSAI parameter invalid
 * \retval Other error CSAI mac generate operation return code
 */
FUNC(ecy_hsm_Csai_ErrorT, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_ProcessJob_SigGenerate(
    CONST(ecy_hsm_Csai_SessionHandleT, AUTOMATIC) hSession,
    CONST(ecy_hsm_Csai_KeyHandleT, AUTOMATIC) hKey,
    CONSTP2VAR(ecy_hsm_Csai_JobHandleT, AUTOMATIC, CRYPTO_11_HSM_VAR) phJob,
    CONST(ecy_hsm_Csai_JobPriorityT, AUTOMATIC) priority,
    CONSTP2CONST(Crypto_JobType, AUTOMATIC, CRYPTO_11_HSM_CONST) job,
    CONSTP2VAR(Crypto_11_Hsm_ServiceData, AUTOMATIC, CRYPTO_11_HSM_VAR) data,
    CONST(boolean, AUTOMATIC) finishAfterUpdate
);
#else
#define Crypto_11_Hsm_ProcessJob_SigGenerate    Crypto_11_Hsm_ProcessJob_Dummy
#endif

#if (CRYPTO_11_HSM_CONFIG_HAS_RANDOMGENERATE == STD_ON)
/**
 * \req [$DD 9]
 * \req [$Satisfies $SWS_Crypto 00071][$Satisfies $SWS_Crypto 00073][$Satisfies $SWS_Crypto 00135][$Satisfies $SWS_Crypto 91003]
 * \req [$Satisfies $ARCH RNG_1][$Satisfies $ARCH JOBS_19]
 *
 * \brief Performs the random number generation primitive, that is configured in the job parameter.
 *
 * This function interfaces with the CycurHSM function ecy_hsm_Csai_RandomGenerate in order to
 * generate the required pseudo random binary number.
 * This sequence is seeded at initialisation and periodically with a true random number.
 *
 * \param[in] hSession HSM session corresponding to the crypto driver object
 * \param[in] hKey Set to NULL_PTR for this service
 * \param[in,out] phJob Pointer to the HSM job executing the service
 * \param[in] priority Job priority
 * \param[in] job Crypto Driver Object job under processing
 * \param[in] data Input data to the service
 * \param[in,out] finishAfterUpdate Not used for this service
 * \retval ecy_hsm_CSAI_SUCCESS Request successful
 * \retval ecy_hsm_CSAI_ERR_INVALID_HANDLE The given session handle is invalid
 * \retval ecy_hsm_CSAI_FC_BUSY Another job is still pending completion in this session
 */
FUNC(ecy_hsm_Csai_ErrorT, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_ProcessJob_RandomGenerate(
    CONST(ecy_hsm_Csai_SessionHandleT, AUTOMATIC) hSession,
    CONST(ecy_hsm_Csai_KeyHandleT, AUTOMATIC) hKey,
    CONSTP2VAR(ecy_hsm_Csai_JobHandleT, AUTOMATIC, CRYPTO_11_HSM_VAR) phJob,
    CONST(ecy_hsm_Csai_JobPriorityT, AUTOMATIC) priority,
    CONSTP2CONST(Crypto_JobType, AUTOMATIC, CRYPTO_11_HSM_CONST) job,
    CONSTP2VAR(Crypto_11_Hsm_ServiceData, AUTOMATIC, CRYPTO_11_HSM_VAR) data,
    CONST(boolean, AUTOMATIC) finishAfterUpdate
);
#else
#define Crypto_11_Hsm_ProcessJob_RandomGenerate     Crypto_11_Hsm_ProcessJob_Dummy
#endif

/**
 * \req [$DD 97]
 * \req [$Satisfies $ARCH JOBS_19][$Satisfies $ARCH SHE_10]
 * \brief Dispatches the crypto job to the underlying service processing function
 */
FUNC(ecy_hsm_Csai_ErrorT, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_DispatchJobForProcessing(
    CONSTP2CONST(Crypto_JobType, AUTOMATIC, CRYPTO_11_HSM_CONST) job,
    CONST(uint32, AUTOMATIC) objId,
    CONST(ecy_hsm_Csai_SessionHandleT, AUTOMATIC) sessionHandle,
    CONSTP2VAR(ecy_hsm_Csai_JobHandleT, AUTOMATIC, CRYPTO_11_HSM_VAR) pJobHandle,
    CONST(boolean, AUTOMATIC) keyLocked,
    CONSTP2VAR(Crypto_11_Hsm_ServiceData, AUTOMATIC, CRYPTO_11_HSM_VAR) objDataPtr,
    CONST(boolean, AUTOMATIC) finishAfterUpdate
);

/**
 * \req [$DD 98]
 * \req [$Satisfies $ARCH JOBS_19][$Satisfies $ARCH SHE_10]
 * \brief Dispaches the crypto job to the underlying service post processing function
 */
FUNC(void, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_DispatchPostProcessing(
    CONST(uint32, AUTOMATIC) objId,
    CONSTP2VAR(ecy_hsm_Csai_ErrorT, AUTOMATIC, AUTOMATIC) pError,
    CONSTP2CONST(Crypto_JobType, AUTOMATIC, CRYPTO_11_HSM_CONST) job,
    CONSTP2CONST(Crypto_11_Hsm_ServiceData, AUTOMATIC, CRYPTO_11_HSM_VAR) objDataPtr
);


/**
 * \req [$DD 99]
 * \req [$Satisfies $ARCH JOBS_4][$Satisfies $ARCH JOBS_11]
 * \brief Does the polling of an ongoing CSAI or SHE operation on the given crypto object
 */
FUNC(ecy_hsm_Csai_ErrorT, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_PollJobHandle(
    CONST(uint32, AUTOMATIC) objId,
    CONST(ecy_hsm_Csai_JobHandleT, AUTOMATIC) jobHandle
);

/**
 * \req [$DD 94]
 * \req [$Satisfies $ARCH JOBS_17]
 * \req [$Satisfies $ARCH SHE_9][$Satisfies $ARCH SHE_15]
 * \brief Cancels ongoing SHE or CSAI operation
 */
FUNC(ecy_hsm_Csai_ErrorT, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_CancelOperation(
    CONST(uint32, AUTOMATIC) objectId,
    CONST(ecy_hsm_Csai_SessionHandleT, AUTOMATIC) hSession
);

/**
 * \req [$DD 100]
 * \req [$Satisfies $ARCH JOBS_18]
 * \brief Cancels streaming operation
 */
FUNC(ecy_hsm_Csai_ErrorT, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_CancelStreamingOperation(
    CONST(uint32, AUTOMATIC) objectId,
    CONST(ecy_hsm_Csai_JobHandleT, AUTOMATIC) hJob);

/**
 * \req [$DD 101]
 * \req [$Satisfies $SWS_Crypto 00072]
 * \brief Checks if the given crypto service require a key
 */
FUNC(boolean, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_ServiceNeedsKey(
    CONSTP2CONST(Crypto_JobType, AUTOMATIC, CRYPTO_11_HSM_CONST) job
);

/**
 * \req [$DD 102]
 * \req [$Satisfies $SWS_Crypto 00039][$Satisfies $ARCH KEYM_9][$Satisfies $ARCH KEYM_10][$Satisfies $ARCH KEYM_11][$Satisfies $ARCH KEYM_12][$Satisfies $ARCH KEYM_13][$Satisfies $ARCH KEYM_14][$Satisfies $ARCH KEYM_18]
 * \req [$Satisfies $ARCH KEYM_24][$Satisfies $ARCH KEYM_25][$Satisfies $ARCH KEYM_26][$Satisfies $ARCH KEYM_27][$Satisfies $ARCH KEYM_28][$Satisfies $ARCH KEYM_30][$Satisfies $ARCH KEYM_32][$Satisfies $ARCH KEYM_33][$Satisfies $ARCH KEYM_49][$Satisfies $ARCH KEYM_50]
 * \brief Requests the key loader to start and process key loading operation
 */
FUNC(Std_ReturnType, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_LoadCryptoKey(
    CONST(uint32, AUTOMATIC) objId,
    CONST(ecy_hsm_Csai_SessionHandleT, AUTOMATIC) sessionHandle,
    CONSTP2CONST(Crypto_JobType, AUTOMATIC, CRYPTO_11_HSM_CONST) job
);

/**
 * \req [$DD 95]
 * \req [$Satisfies $ARCH JOBS_1][$Satisfies $ARCH SHE_27]
 * \brief Opens a CSAI session depending on the crypto object
 *
 * \param[in] objId       - ID of Crypto Driver object to open session with
 * \param[in] hSessionPtr - pointer to session handle to write to
 */
FUNC(ecy_hsm_Csai_ErrorT, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_OpenSession(
    CONST(uint32, AUTOMATIC) objId,
    CONSTP2VAR(ecy_hsm_Csai_SessionHandleT, AUTOMATIC, CRYPTO_11_HSM_VAR) hSessionPtr
);

/**
 * \req [$DD 103]
 * \req [$Satisfies $ARCH KEYM_26][$Satisfies $ARCH KEYM_27][$Satisfies $ARCH JOBS_13]
 * \brief Maps crypto job priority to CSAI job priority
 */
FUNC(ecy_hsm_Csai_JobPriorityT, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_MapJobPriority(CONST(uint32, AUTOMATIC) jobPriority);

/**
 * \req [$DD 84]
 * \req [$Satisfies $ARCH SHE_2][$Satisfies $ARCH SHE_3][$Satisfies $ARCH SHE_8][$Satisfies $ARCH SHE_9]$Satisfies $ARCH JOBS_16]
 *
 * \brief  Returns timeout of passed job which executes in specific object
 *
 * \param[in]  job                Job for which the timeout is selected
 * \param[in]  objectId           Object id in which the job is being processed.
 * \param[out] pTimeoutUnit       Timeout unit to be used for this job.  Can return either  CRYPTO_11_HSM_TICKS
 *                                or CRYPTO_11_HSM_MS
 * \retval                        Timeout value
 **/
FUNC(uint32, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_GetJobTimeout(
    CONSTP2CONST(Crypto_JobType, AUTOMATIC, CRYPTO_11_HSM_CONST) job,
    CONST(uint32, AUTOMATIC) objectId,
    CONSTP2VAR(Crypto_11_Hsm_TimerUnitT, AUTOMATIC, AUTOMATIC) pTimeoutUnit);

/**
 * \req [$DD 104]
 * \req [$Satisfies $ARCH SHE_9][$Satisfies $ARCH SHE_15]
 * \brief Synchronously cancel running SHE job otherwise return ecy_hsm_CSAI_ERR_NOT_SUPPORTED.
 **/
FUNC(ecy_hsm_Csai_ErrorT, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_Sync_Cancel(CONST(uint32, AUTOMATIC) objectId);

/**
 * \req [$DD 105]
 * \req [$Satisfies $ARCH JOBS_3][$Satisfies $ARCH JOBS_10]
 * \brief Performs post processing of not scheduled job
 */
FUNC(Std_ReturnType, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_ProcessJob_NotScheduled(
    CONST(uint32, AUTOMATIC) objectId,
    CONSTP2CONST(Crypto_JobType, AUTOMATIC, CRYPTO_11_HSM_CONST) job);

#if (CRYPTO_11_HSM_CONFIG_HAS_SHE_SERVICES == STD_ON)
/**
 * \req [$DD 74]
 * \req [$Satisfies $ARCH SHE_25]
 *
 * \brief Performs Random Number Generation primitive using SHE interface
 *
 * \internal
 * This function processes pseudorandom number generation calls through the SHE interface.
 * Due to the asynchronous design nature of this SHE interface,
 * the result will not be available when the function returns.
 * Use Crypto_11_Hsm_SHE_Sync to sync the result.
 * The function converts the SHE error code into CSAI error code.
 * The SHE interface does not support streaming execution.
 *
 * \note
 * As per the SHE specification, CMD_RND generates exactly 16 bytes.
 * This size cannot be configured or altered.
 * The user is expected to provide a buffer of size <= 16 bytes.
 *
 * \param[in]     keyId - SHE keyId to use. Unused for RNG
 * \param[in,out] job   - Crypto Driver Object job under processing
 * \param[in,out] data  - data input or output from the service
 *
 * \retval ecy_hsm_CSAI_ERR_INVALID_PARAM    - Incorrect parameter supplied
 * \retval ecy_hsm_CSAI_ERR_BUFFER_TOO_SMALL - Buffer size must be >= 16 bytes
 * \retval CSAI_ERR_SHE_ERC_GENERAL_ERROR    - General failure
 * \retval CSAI_ERR_SHE_ERC_NO_ERROR         - Success
 */
FUNC(ecy_hsm_Csai_ErrorT, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_ProcessJob_SHE_RandomGenerate(
    CONST(SHE_KeyIdT, AUTOMATIC) keyId,
    CONSTP2CONST(Crypto_JobType, AUTOMATIC, CRYPTO_11_HSM_CONST) job,
    CONSTP2VAR(Crypto_11_Hsm_ServiceData, AUTOMATIC, CRYPTO_11_HSM_VAR) data
);

/**
 * \req [$DD 68]
 * \req [$Satisfies $ARCH SHE_20][$Satisfies $SWS_Crypto 00072]
 * \brief Performs SHE CMAC Verification
 *
 * \internal
 * This is a wrapper function to SHE_VerifyMAC interface that performs a CMAC verification using AES-128.
 * Due to the asynchronous design nature of this SHE interface, the result will
 * not be available when the function returns. Use Crypto_11_Hsm_SHE_Sync to sync the result.
 * The function converts the SHE error code into CSAI error code.
 * The SHE interface does not support streaming execution.
 *
 * \param[in]     keyId - SHE keyId to use.
 * \param[in]     job   - Crypto Driver Object job under processing
 * \param[in,out] data  - data input or output from the service
 *
 * \retval ecy_hsm_CSAI_ERR_INVALID_PARAM     - Incorrect parameter supplied
 * \retval CSAI_ERR_SHE_ERC_KEY_NOT_AVAILABLE - Supplied key is locked
 * \retval CSAI_ERR_SHE_ERC_KEY_EMPTY         - Supplied key was not initialised
 * \retval CSAI_ERR_SHE_ERC_KEY_INVALID       - Supplied key does not have the required properties
 * \retval CSAI_ERR_SHE_ERC_GENERAL_ERROR     - General error
 * \retval CSAI_ERR_SHE_ERC_NO_ERROR          - Success
 */
FUNC(ecy_hsm_Csai_ErrorT, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_ProcessJob_SHE_MacVerify(
    CONST(SHE_KeyIdT, AUTOMATIC) keyId,
    CONSTP2CONST(Crypto_JobType, AUTOMATIC, CRYPTO_11_HSM_CONST) job,
    CONSTP2VAR(Crypto_11_Hsm_ServiceData, AUTOMATIC, CRYPTO_11_HSM_VAR) data
);

/**
 * \req [$DD 69]
 * \req [$Satisfies $ARCH SHE_19][$Satisfies $SWS_Crypto 00072]
 * \brief Performs SHE CMAC generation
 *
 * \internal
 * This is a wrapper function to SHE_GenerateMAC interface that performs a CMAC generation using AES-128.
 * Due to the asynchronous design nature of this SHE interface, the result will
 * not be available when the function returns. Use Crypto_11_Hsm_SHE_Sync to sync the result.
 * The function converts the SHE error code into CSAI error code.
 * The SHE interface does not support streaming execution.
 *
 * \param[in]     keyId - SHE keyId to use.
 * \param[in]     job   - Crypto Driver Object job under processing
 * \param[in,out] data  - data input or output from the service
 *
 * \retval ecy_hsm_CSAI_ERR_INVALID_PARAM     - Incorrect parameter supplied
 * \retval CSAI_ERR_SHE_ERC_NO_ERROR          - Success
 * \retval CSAI_ERR_SHE_ERC_KEY_NOT_AVAILABLE - Supplied key is locked
 * \retval CSAI_ERR_SHE_ERC_KEY_EMPTY         - Supplied key was not initialised
 * \retval CSAI_ERR_SHE_ERC_KEY_INVALID       - Supplied key does not have the required properties
 * \retval CSAI_ERR_SHE_ERC_GENERAL_ERROR     - General error
 */
FUNC(ecy_hsm_Csai_ErrorT, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_ProcessJob_SHE_MacGenerate(
    CONST(SHE_KeyIdT, AUTOMATIC) keyId,
    CONSTP2CONST(Crypto_JobType, AUTOMATIC, CRYPTO_11_HSM_CONST) job,
    CONSTP2VAR(Crypto_11_Hsm_ServiceData, AUTOMATIC, CRYPTO_11_HSM_VAR) data
);

/**
 * \req [$DD 70]
 * \req [$Satisfies $ARCH SHE_20]
 * \brief Performs postprocessing operation on SHE CMAC verification
 *
 * \internal
 * This function updates the result of the SHE CMAC verification to the job verification pointer
 *
 * \param[in]     pError - Pointer to the error code from the service processing
 * \param[in]     job    - Crypto Driver Object job under processing
 * \param[in/out] data   - data input or output from the service
 */
FUNC(void, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_PostProcessing_SHE_MacVerify(
    CONSTP2CONST(ecy_hsm_Csai_ErrorT, AUTOMATIC, AUTOMATIC) pError,
    CONSTP2CONST(Crypto_JobType, AUTOMATIC, CRYPTO_11_HSM_CONST) job,
    CONSTP2CONST(Crypto_11_Hsm_ServiceData, AUTOMATIC, CRYPTO_11_HSM_CONST) data
);


/**
 * \req [$DD 89]
 * \req [$Satisfies $ARCH SHE_21][$Satisfies $ARCH SHE_22]
 * \brief Processes SHE AES encryption job in either ECB or CBC mode
 *
 * \details
 * SHE supports two AES encryption modes
 *      CBC - Chained Block Cipher
 *          - requires an initialisation vector (must be 16 bytes)
 *          - plaintext input must be (n x 16) bytes long
 *      ECB - Electronic Code Book
 *          - plaintext input must be exactly 16 bytes long
 *     For both modes, the output buffer must be at least as large as the input message
 *
 * \note
 * The Crypto Driver is not responsible for applying padding to
 * input or output vectors for SHE encryption.
 * All padding must be handled by the user application.
 *
 * \internal
 * Due to the asynchronous design nature of this SHE interface,
 * the result will not be available when the function returns.
 * Use Crypto_11_Hsm_SHE_Sync to sync the result.
 * The function converts the SHE error code into CSAI error code.
 * The SHE interface does not support streaming execution.
 *
 * \param[in]     keyId  - SHE key ID to be used
 * \param[in]     job    - Crypto Driver Object job under processing
 * \param[in,out] data   - Data input or output from the service.
 *                         Contains IV for CBC encryption
 */
FUNC(ecy_hsm_Csai_ErrorT, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_ProcessJob_SHE_Encrypt(
    CONST(SHE_KeyIdT, AUTOMATIC) keyId,
    CONSTP2CONST(Crypto_JobType, AUTOMATIC, CRYPTO_11_HSM_CONST) job,
    CONSTP2VAR(Crypto_11_Hsm_ServiceData, AUTOMATIC, CRYPTO_11_HSM_VAR) data
);

/**
 * \req [$DD 90]
 * \req [$Satisfies $ARCH SHE_23][$Satisfies $ARCH SHE_24]
 * \brief Processes SHE AES decryption job in either ECB or CBC mode
 *
 * \details
 * SHE supports two AES decryption modes
 *      CBC - Chained Block Cipher
 *          - requires an initialisation vector (must be 16 bytes)
 *          - ciphertext input must be (n x 16) bytes long
 *      ECB - Electronic Code Book
 *          - ciphertext input must be exactly 16 bytes long
 *
 * \note
 * The Crypto Driver is not responsible for applying padding to
 * input or output vectors for SHE decryption.
 * All padding must be handled by the user application.
 *
 * \internal
 * Due to the asynchronous design nature of this SHE interface,
 * the result will not be available when the function returns.
 * Use Crypto_11_Hsm_SHE_Sync to sync the result.
 * The function converts the SHE error code into CSAI error code.
 * The SHE interface does not support streaming execution.
 *
 * \param[in]     keyId  - SHE key ID to be used
 * \param[in]     job    - Crypto Driver Object job under processing
 * \param[in,out] data   - Data input or output from the service.
 *                         Contains IV for CBC decryption
 */
FUNC(ecy_hsm_Csai_ErrorT, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_ProcessJob_SHE_Decrypt(
    CONST(SHE_KeyIdT, AUTOMATIC) keyId,
    CONSTP2CONST(Crypto_JobType, AUTOMATIC, CRYPTO_11_HSM_CONST) job,
    CONSTP2VAR(Crypto_11_Hsm_ServiceData, AUTOMATIC, CRYPTO_11_HSM_VAR) data
);

/**
 * \req [$DD 72]
 * \req [$Satisfies $ARCH JOBS_4][$Satisfies $ARCH JOBS_11]
 *
 * \brief Polls the result of a running SHE operation
 *
 * This is a wrapper function to SHE_Sync that synchronize the results of a running SHE functionality.
 * The function converts the SHE error code into CSAI error code.
 *
 * \retval CSAI_ERR_SHE_ERC_NO_ERROR       - No error
 * \retval CSAI_ERR_SHE_ERC_BUSY           - HSM is still busy
 * \retval CSAI_ERR_SHE_ERC_MEMORY_FAILURE - A memory error occurred
 * \retval CSAI_ERR_SHE_ERC_GENERAL_ERROR  - General error
 */
FUNC(ecy_hsm_Csai_ErrorT, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_SHE_Sync(void);

/**
 * \req [$DD 85]
 * \req [$Satisfies $ARCH SHE_9][$Satisfies $ARCH SHE_15]
 * \brief Synchronously cancels running SHE operation
 *
 * \retval CSAI_ERR_SHE_ERC_NO_ERROR   Operation cancelled or how swap performed. SHE can accept further requests.
 * \retval CSAI_ERR_SHE_ERC_BUSY       HSM cannot cancel currently running operation because hot swap conditions are not
 *                                     met.
 * \retval others                      Indicates what error occurred.
 */
FUNC(ecy_hsm_Csai_ErrorT, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_SHE_Cancel(void);

/**
 * \req [$DD 73]
 * \req [$Satisfies $ARCH SHE_27]
 *
 * \brief Returns a pointer to the SHE session handle.
 * \retval ecy_hsm_Csai_SessionHandleT pointer to the SHE session handle
 */
FUNC(P2VAR(ecy_hsm_Csai_SessionHandleT, AUTOMATIC, AUTOMATIC), CRYPTO_11_HSM_CODE) Crypto_11_Hsm_SHE_GetSessionPtr(void);

#else
/**
 * \brief Dummy function to handle all SHE services when SHE is disabled.
 */
FUNC(ecy_hsm_Csai_ErrorT, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_SHE_Dummy(
    CONST(SHE_KeyIdT, AUTOMATIC) keyId,
    CONSTP2CONST(Crypto_JobType, AUTOMATIC, CRYPTO_11_HSM_CONST) job,
    CONSTP2VAR(Crypto_11_Hsm_ServiceData, AUTOMATIC, CRYPTO_11_HSM_VAR) data
);

/**
 * \brief Dummy function to all handle postprocessing when SHE is disabled.
 */
FUNC(void, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_SHE_PostProcess_Dummy(
    CONSTP2CONST(ecy_hsm_Csai_ErrorT, AUTOMATIC, AUTOMATIC) pError,
    CONSTP2CONST(Crypto_JobType, AUTOMATIC, CRYPTO_11_HSM_CONST) job,
    CONSTP2CONST(Crypto_11_Hsm_ServiceData, AUTOMATIC, CRYPTO_11_HSM_CONST) data
);

#define Crypto_11_Hsm_ProcessJob_SHE_RandomGenerate  Crypto_11_Hsm_SHE_Dummy
#define Crypto_11_Hsm_ProcessJob_SHE_MacVerify       Crypto_11_Hsm_SHE_Dummy
#define Crypto_11_Hsm_ProcessJob_SHE_MacGenerate     Crypto_11_Hsm_SHE_Dummy
#define Crypto_11_Hsm_PostProcessing_SHE_MacVerify   Crypto_11_Hsm_SHE_PostProcess_Dummy
#define Crypto_11_Hsm_ProcessJob_SHE_Encrypt         Crypto_11_Hsm_SHE_Dummy
#define Crypto_11_Hsm_ProcessJob_SHE_Decrypt         Crypto_11_Hsm_SHE_Dummy
#endif  /* CRYPTO_11_HSM_CONFIG_HAS_SHE_SERVICES */

#if (CRYPTO_11_HSM_CONFIG_HAS_SHE_SERVICES == STD_ON) || (CRYPTO_11_HSM_CONFIG_HAS_SHE_KEY_MGMT == STD_ON)
/**
 * \req [$DD 71]
 * \req [$Satisfies $ARCH SHE_27]
 *
 * \brief Opens a SHE Session
 *
 * This is a wrapper function to SHE_Init that opens a SHE session and
 * converts the SHE error code into CSAI error code.
 *
 * \retval CSAI_ERR_SHE_ERC_NO_ERROR - No error
 * \retval CSAI_ERR_SHE_ERC_GENERAL_ERROR - General error
 * \retval CSAI_ERR_SHE_ERC_SEQUENCE_ERROR - Session already open
 */
FUNC(ecy_hsm_Csai_ErrorT, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_SHE_Init(void);

/**
 * \req [$DD 108]
 * \req [$Satisfies $ARCH CONFIG_7]
 *
 * \brief Closes a SHE Session
 *
 * This is a wrapper function to SHE_DeInit that closes a SHE session and
 * converts the SHE error code into CSAI error code.
 *
 * \retval CSAI_ERR_SHE_ERC_NO_ERROR - No error
 * \retval CSAI_ERR_SHE_ERC_GENERAL_ERROR - General error
 * \retval CSAI_ERR_INVALID_HANDLE - The call was not successful. There was no open SHE session.
 */
FUNC(ecy_hsm_Csai_ErrorT, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_SHE_DeInit(void);

/**
 * \req [$DD 93]
 * \req [$Satisfies $ARCH SHE_1]
 *
 * \brief Sends a callback notification to BswM, if the error code of the last SHE operation has changed.
 * First notification will be send after opening the SHE session
 *
 * \param[in] errorCode - CSAI error code of the last SHE operation
 */
FUNC(void, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_SHE_SendCbkNotificationIfRequired(CONST(ecy_hsm_Csai_ErrorT, AUTOMATIC) errorCode);


/**
 * \req [$DD 92]
 * \req [$Satisfies $ARCH SHE_1]
 *
 * \brief Translates standard AUTOSAR error code into CSAI SHE compatible error code
 *
 * \param[in] stdError - Std_ReturnType error code
 * \retval ecy_hsm_Csai_ErrorT SHE compatible error code
 */
FUNC(ecy_hsm_Csai_ErrorT, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_TranslateStdErrorIntoSheError(CONST(Std_ReturnType, AUTOMATIC) stdError);

#endif /* CRYPTO_11_HSM_CONFIG_HAS_SHE_KEY_MGMT or CRYPTO_11_HSM_CONFIG_HAS_SHE_KEY_MGMT */

#define CRYPTO_11_HSM_STOP_SEC_CODE
#include <Crypto_11_Hsm_MemMap.h>

#endif
