/**
 * \file
 * \copyright (c) ESCRYPT GmbH, Bochum, Germany 2019. All rights reserved, also regarding any disposal, exploitation,
 * reproduction, editing, distribution, as well as in the event of applications for industrial property rights.
 */

/* Global QA-C suppression */
/* PRQA S 0776, 0778 EOF #Deviation ID directive_1.1_0776_0778 */
/* PRQA S 0292 EOF #Deviation ID directive_1.1_0292 */
/* PRQA S 0857 EOF #Deviation ID directive_1.1_0857 */
/* PRQA S 2053 EOF #Deviation ID rule_18.8_2053 */
/* PRQA S 3214 EOF #Suppressed because of the ASR memmap concept */
/* PRQA S 2000 EOF #Suppressed for simplicity. Not necessary having else statement is allowed. */
/* PRQA S 5002 EOF #Suppressed as header file names are defined by ASR. */

#ifndef CRYPTO_11_HSM_TIMER_H
#define CRYPTO_11_HSM_TIMER_H

#include <Std_Types.h>

typedef enum
{
    CRYPTO_11_HSM_MS,
    CRYPTO_11_HSM_TICKS
} Crypto_11_Hsm_TimerUnitT;

/**
 * \req [$DD 86]
 * \req [$Satisfies $ARCH CONFIG_6][$Satisfies $ARCH JOBS_16][$Satisfies $ARCH KEYM_42]
 * \req [$Satisfies $ARCH SHE_2][$Satisfies $ARCH SHE_3][$Satisfies $ARCH SHE_4][$Satisfies $ARCH SHE_5][$Satisfies $ARCH SHE_6][$Satisfies $ARCH SHE_7][$Satisfies $ARCH SHE_8]
 *
 * \brief Initialize timer referenced by timerId
 * 
 * \param[in]  timerId                 Unique reference to allocated timer
 *                                
 * \retval                             E_NOT_OK   Timer cannot be allocated
 * \retval                             E_OK       Timer allocated, reference returned 
 **/
FUNC(void, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_InitTimer(CONST(uint32, AUTOMATIC) timerId);

/**
 * \req [$DD 87]
 * \req [$Satisfies $ARCH CONFIG_6][$Satisfies $ARCH JOBS_16][$Satisfies $ARCH KEYM_42]
 * \req [$Satisfies $ARCH SHE_2][$Satisfies $ARCH SHE_3][$Satisfies $ARCH SHE_4][$Satisfies $ARCH SHE_5][$Satisfies $ARCH SHE_6][$Satisfies $ARCH SHE_7][$Satisfies $ARCH SHE_8]
 * 
 * \brief Set expiry time for timer referenced by timerId
 * 
 * \param[in]  timeout                 Timeout, expiry time
 * \param[in]  timerId                 Unique reference to allocated timer
 *
 **/
FUNC(void, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_SetExpiryTime(CONST(uint32, AUTOMATIC) timerId, CONST(uint32, AUTOMATIC) timeout);

/**
 * \req [$DD 88]
 * \req [$Satisfies $ARCH CONFIG_6][$Satisfies $ARCH JOBS_16][$Satisfies $ARCH KEYM_42]
 * \req [$Satisfies $ARCH SHE_2][$Satisfies $ARCH SHE_3][$Satisfies $ARCH SHE_4][$Satisfies $ARCH SHE_5][$Satisfies $ARCH SHE_6][$Satisfies $ARCH SHE_7][$Satisfies $ARCH SHE_8]
 *
 * \brief Start monitoring time. The caller checks with Crypto_11_Hsm_DidTimerExpire whether the timeout expired
 * \note  It is expected that the timeout was previously set, otherwise the timer cannot expire
 * 
 * \param[in]  timerId                 Timeout, expiry time
 *
 **/
FUNC(void, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_StartTimer(CONST(uint32, AUTOMATIC) timerId);

/**
 * \req [$DD 79]
 * \req [$Satisfies $ARCH CONFIG_6][$Satisfies $ARCH JOBS_16][$Satisfies $ARCH KEYM_42]
 * \req [$Satisfies $ARCH SHE_2][$Satisfies $ARCH SHE_3][$Satisfies $ARCH SHE_4][$Satisfies $ARCH SHE_5][$Satisfies $ARCH SHE_6][$Satisfies $ARCH SHE_7][$Satisfies $ARCH SHE_8]
 *  
 * \brief Check if the timer has expired.
 * \note  It is expected that the timeout was previously set, otherwise the timer cannot expire
 * 
 * \param[in]  timerId                 Unique reference to allocated timer
 * \retval     TRUE                    Expired 
 * \retval     FALSE                   Not expired 
 **/
FUNC(boolean, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_DidTimerExpire(CONST(uint32, AUTOMATIC) timerId);

/**
 * \req [$DD 80]
 * \req [$Satisfies $ARCH CONFIG_6][$Satisfies $ARCH JOBS_16][$Satisfies $ARCH KEYM_42]
 * \req [$Satisfies $ARCH SHE_2][$Satisfies $ARCH SHE_3][$Satisfies $ARCH SHE_4][$Satisfies $ARCH SHE_5][$Satisfies $ARCH SHE_6][$Satisfies $ARCH SHE_7][$Satisfies $ARCH SHE_8]
 *  
 * \brief Combines operations Crypto_11_Hsm_SetExpiryTime and Crypto_11_Hsm_StartTimer in one call
 * \note  It is expected that the timeout was previously set, otherwise the timer cannot expire
 * 
 * \param[in]  timerId                 Unique reference to allocated timer
 * \param[in]  timeout                 Timeout, expiry time
 **/
FUNC(void, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_SetExpiryAndStartTimer(
    CONST(uint32, AUTOMATIC) timerId,
    CONST(uint32, AUTOMATIC) timeout);

/**
 * \req [$DD 83]
 * \req [$Satisfies $ARCH CONFIG_6][$Satisfies $ARCH JOBS_16][$Satisfies $ARCH KEYM_42]
 * \req [$Satisfies $ARCH SHE_2][$Satisfies $ARCH SHE_3][$Satisfies $ARCH SHE_4][$Satisfies $ARCH SHE_5][$Satisfies $ARCH SHE_6][$Satisfies $ARCH SHE_7][$Satisfies $ARCH SHE_8]
 *  
 * \brief Set timer unit
 * 
 * \param[in]  timerId                 Unique reference to allocated timer
 * \param[in]  timerUnit               Unit of time measuring CRYPTO_11_HSM_TICKS or CRYPTO_11_HSM_MS
 **/
/* PRQA S 1503,1505,1532 1 #The API is called externally. Deviation ID rule_2.1_1503 */
FUNC(void, CRYPTO_11_HSM_CODE) Crypto_11_Hsm_SetTimerUnit(
    CONST(uint32, AUTOMATIC) timerId,
    CONST(Crypto_11_Hsm_TimerUnitT, AUTOMATIC) timerUnit);

#endif
